# SPDX-License-Identifier: BSD-2-Clause

from textwrap                           import dedent
from typing                             import Literal

from torii.build                        import Attrs, Clock, Connector, DiffPairs, Pins, PinsN, Resource, Subsignal
from torii.build.run                    import BuildProducts
from torii.platform.resources.interface import UARTResource
from torii.platform.resources.memory    import SPIFlashResources
from torii.platform.resources.user      import ButtonResources, LEDResources, SwitchResources
from torii.platform.vendor.lattice.ecp5 import ECP5Platform

__all__ = (
	'ECP55GEVNPlatform',
)

class ECP55GEVNPlatform(ECP5Platform):
	device      = 'LFE5UM5G-85F'
	package     = 'BG381'
	speed       = '8'
	default_clk = 'clk12'
	default_rst = 'rst'

	pretty_name = 'ECP55GEVN'
	description = 'ECP55GEVN Lattice ECP5-5G-85F Evaluation Board'

	def __init__(
		self, *, VCCIO1: Literal['3V3', '2V5'] = '2V5', VCCIO6: Literal['3V3', '2V5'] = '3V3', **kwargs
	) -> None:
		'''
		VCCIO1 is connected by default to 2.5 V via R100 (can be set to 3.3 V by disconnecting
		R100 and connecting R105)
		VCCIO6 is connected to 3.3 V by default via R99 (can be switched to 2.5 V with R104,
		see page 51 in the ECP5-5G-EVN datasheet)
		'''
		super().__init__(**kwargs)
		if VCCIO1 not in ('3V3', '2V5'):
			raise ValueError(f'VCCIO1 is expected to be \'3V3\' or \'2V5\' not {VCCIO1!r}')
		if VCCIO6 not in ('3V3', '2V5'):
			raise ValueError(f'VCCIO6 is expected to be \'3V3\' or \'2V5\' not {VCCIO6!r}')

		self._VCCIO1 = VCCIO1
		self._VCCIO6 = VCCIO6

	def _vccio_to_iostandard(self, vccio: Literal['3V3', '2V5']) -> str:
		if vccio == '2V5':
			return 'LVCMOS25'
		if vccio == '3V3':
			return 'LVCMOS33'

		raise ValueError(f'vccio is expected to be \'3V3\' or \'2V5\' not {vccio!r}')

	def bank1_iostandard(self) -> str:
		return self._vccio_to_iostandard(self._VCCIO1)

	def bank6_iostandard(self) -> str:
		return self._vccio_to_iostandard(self._VCCIO6)

	resources   = [
		Resource('rst', 0, PinsN('G2', dir = 'i'), Attrs(IO_TYPE = 'LVCMOS33')),
		Resource(
			'clk12', 0, Pins('A10', dir = 'i'), Clock(12e6), Attrs(IO_TYPE = 'LVCMOS33')
		),
		# By default this CLK is not populated, see User Manual Section 4.
		Resource(
			'extclk', 0, Pins('B11', dir = 'i'), Attrs(IO_TYPE = 'LVCMOS33')
		),
		*LEDResources(
			pins = 'A13 A12 B19 A18 B18 C17 A17 B17', invert = True,
			attrs = Attrs(IO_TYPE = bank1_iostandard)
		),
		*ButtonResources(
			pins = 'P4', invert = True, attrs = Attrs(IO_TYPE = bank6_iostandard)
		),
		*SwitchResources(
			pins = {1: 'J1', 2: 'H1', 3: 'K1'}, invert = True,
			attrs = Attrs(IO_TYPE = bank6_iostandard)
		),
		*SwitchResources(
			pins = {4: 'E15', 5: 'D16', 6: 'B16', 7: 'C16', 8: 'A16'}, invert = True,
			attrs = Attrs(IO_TYPE = bank1_iostandard)
		),
		# In order to use the UART as a UART you need to swap two resistor jumpers,
		# and potentially reconfigure the onboard FTDI chip, see section 6.2 in the
		# User Manual for hardware instructions, and
		# https://github.com/trabucayre/fixFT2232_ecp5evn to reconfigure the FTDI.
		UARTResource(
			0,
			rx = 'P2', tx = 'P3',
			attrs = Attrs(IO_TYPE = bank6_iostandard, PULLMODE = 'UP')
		),
		*SPIFlashResources(
			0,
			cs_n = 'R2', clk = 'U3', cipo = 'V2', copi = 'W2', wp_n = 'Y2', hold_n = 'W1',
			attrs = Attrs(IO_TYPE = 'LVCMOS33')
		),
		Resource(
			'serdes', 0,
			Subsignal('tx', DiffPairs('W4', 'W5', dir = 'o')),
			Subsignal('rx', DiffPairs('Y5', 'Y6', dir = 'i')),
		),
		Resource(
			'serdes', 1,
			Subsignal('tx', DiffPairs('W8', 'W9', dir = 'o')),
			Subsignal('rx', DiffPairs('Y7', 'Y8', dir = 'i')),
		),
		Resource(
			'serdes', 2,
			Subsignal('tx', DiffPairs('W13', 'W14', dir = 'o')),
			Subsignal('rx', DiffPairs('Y14', 'Y15', dir = 'i')),
		),
		Resource(
			'serdes', 3,
			Subsignal('tx', DiffPairs('W17', 'W18', dir = 'o')),
			Subsignal('rx', DiffPairs('Y16', 'Y17', dir = 'i')),
		),
		Resource('serdes_clk', 0, DiffPairs('Y11', 'Y12', dir = 'i')),
		Resource('serdes_clk', 1, DiffPairs('Y19', 'W20', dir = 'i')), # 200 MHz
		# TODO: add other resources
	]
	connectors  = [
		# Expansion connectors
		Connector(
			'J', 39,
			'-   -   -   D15 B15 C15 B13 B20 D11 E11 B12 C12 D12 E12 C13 D13 E13 A14 A9  B10 '
			'-   -   -   -   -   -   -   -   E7  -   A11 -   A19 -   -   -   -   -   -   -   '
		),
		Connector(
			'J', 40,
			'K2  -   A15 F1  H2  G1  J4  J5  J3  K3  L4  L5  M4  N5  N4  P5  N3  M3  -   - '
			'K5  -   M5  -   L3  -   N2  M1  L2  -   L1  N1  C14 -   P1  E14 D14 -   K4  - '
			'-   -   - '
		),
		# Arduino
		Connector('J', 6, 'K16 J16 H17 J17 H18 H16 - G18 G16 F17'),
		Connector('J', 3, 'F19 F20 E20 E19 D19 D20 C20 K17'),
		Connector('J', 7, 'C18 - D17 - - - - -'),
		Connector('J', 4, 'F18 E17 E18 D18 F16 E16'),
		# Raspberry Pi
		Connector(
			'JP', 8,
			'-   -   T17 -   U16 -   U17 P18 -   N20 N19 T16 M18 -   N17 P17 -   M17 U20 -  '
			'T19 N18 R20 U19 -   R18 L18 L17 U18 -   T18 T20 P20 -   R17 P19 N16 P16 -   R16'
		),
		# GPIO, J5 Contains 4 differential pairs
		Connector('J', 5, '- - H20 G19 - - K18 J18 - - K19 J19 - - K20 J20 - - G20 - - -'),
		Connector('J', 8, '- - L19 M19 L20 M20 L16 -'),
		Connector(
			'J', 32,
			'- - - - A5 A4 - - C5 B5 - - B4 C4 - - B3 A3 - - D5 E4 - - D3 C3 - - E3 F4 - - F5 E5 - - B1 A2 - -'
		), # Contains 9 differential pairs
		Connector(
			'J', 33,
			'- - - - C2 B2 - - D1 C1 - - E1 D2 - - G5 H4 - - H3 H5 - - F3 G3 - - E2 F2 - -'
		), # Contains 7 differential pairs
		# Mic
		Connector('J', 30, '- B6 D9 C9 E9 D10 A6 E10 - -'),
		# PMOD
		Connector('J', 31, 'C6 C7 E8 D8 - - C8 B8 A7 A8 - -'),
		# JTAG
		Connector('J', 1, '- V4 R5 - - U5 - T5'),
		# Parallel configuration, Connect pin 2 / R2 with jumper when needed
		Connector('J', 38, 'W3 R2 T3 Y3 R1 V3 T1 V2 U1 W2 V1 T2 W1 U2 Y2 R2 U3 R3 - -'),
	]

	@property
	def file_templates(self) -> dict[str, str]:
		return {
			**super().file_templates,
			'{{name}}-openocd.cfg': dedent(r'''
			interface ftdi
			ftdi_device_desc 'Lattice ECP5 Evaluation Board'
			ftdi_vid_pid 0x0403 0x6010
			ftdi_channel 0
			ftdi_layout_init 0xfff8 0xfffb
			reset_config none
			adapter_khz 25000

			jtag newtap ecp5 tap -irlen 8 -expected-id 0x81113043
			''')
		}

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		openocd = environ.get('OPENOCD', 'openocd')
		with products.extract(f'{name}-openocd.cfg', f'{name}.svf') as files:
			(config_filename, vector_filename) = files
			check_call([
				openocd,
				'-f', config_filename,
				'-c', f'transport select jtag; init; svf -quiet {vector_filename}; exit'
			])


if __name__ == '__main__':
	from ...test.blinky import Blinky
	ECP55GEVNPlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                        import Attrs, Clock, Connector, DiffPairs, Pins, PinsN, Resource, Subsignal
from torii.build.run                    import BuildPlan, BuildProducts
from torii.hdl.ir                       import Fragment
from torii.platform.resources.interface import DirectUSBResource
from torii.platform.resources.memory    import SDCardResources, SPIFlashResources
from torii.platform.resources.user      import ButtonResources, RGBLEDResource
from torii.platform.vendor.lattice.ecp5 import ECP5Platform

# NOTE: Keep OrangeCrabR0_2FPlatform for backwards compatibility
# Originally, there was only OrangeCrabR0_2FPlatform, but the 85F variant
# needs a different device.
__all__ = (
	'OrangeCrabR0_2Platform',
	'OrangeCrabR0_2_25FPlatform',
	'OrangeCrabR0_2_85FPlatform',
)

class OrangeCrabR0_2Platform(ECP5Platform):
	device      = 'LFE5U-25F'
	package     = 'MG285'
	speed       = '8'
	default_clk = 'clk'

	pretty_name = 'OrangeCrab R0.2'
	description = 'OrangeCrab R0.2 Lattice ECP5-25F Development Board'

	resources   = [
		Resource('clk', 0, Pins('A9', dir = 'i'), Clock(48e6), Attrs(IO_TYPE = 'LVCMOS33')),
		# Used to reload FPGA configuration.
		# Can enter USB bootloader by assigning button 0 to program.
		Resource('program', 0, PinsN('V17', dir = 'o'), Attrs(IO_TYPE = 'LVCMOS33')),
		RGBLEDResource(
			0,
			r = 'K4', g = 'M3', b = 'J3', invert = True,
			attrs = Attrs(IO_TYPE = 'LVCMOS33')
		),
		*ButtonResources(
			pins = {0: 'J17' }, invert = True,
			attrs = Attrs(IO_TYPE = 'SSTL135_I')
		),
		*SPIFlashResources(
			0,
			cs_n = 'U17', clk = 'U16', cipo = 'T18', copi = 'U18', wp_n = 'R18', hold_n = 'N18',
			attrs = Attrs(IO_TYPE = 'LVCMOS33'),
		),
		# TODO(aki): Replace with `DDR3Resource` when more granular attrs are possible
		Resource(
			'ddr3', 0,
			Subsignal('rst', PinsN('L18', dir = 'o')),
			Subsignal('clk', DiffPairs('J18', 'K18', dir = 'o'), Attrs(IO_TYPE = 'SSTL135D_I')),
			Subsignal('clk_en', Pins('D18', dir = 'o')),
			Subsignal('cs', PinsN('A12', dir = 'o')),
			Subsignal('we', PinsN('B12', dir = 'o')),
			Subsignal('ras', PinsN('C12', dir = 'o')),
			Subsignal('cas', PinsN('D13', dir = 'o')),
			Subsignal('a', Pins('C4 D2 D3 A3 A4 D4 C3 B2 B1 D1 A7 C2 B6 C1 A2 C7', dir = 'o')),
			Subsignal('ba', Pins('P5 N3 M3', dir = 'o')),
			Subsignal(
				'dqs',
				DiffPairs('G18 H17', 'B15 A16', dir = 'io'),
				Attrs(IO_TYPE = 'SSTL135D_I', TERMINATION = 'OFF', DIFFRESISTOR = '100')
			),
			Subsignal(
				'dq',
				Pins('C17 D15 B17 C16 A15 B13 A17 A13 F17 F16 G15 F15 J16 C18 H16 F18', dir = 'io'),
				Attrs(TERMINATION = '75')
			),
			Subsignal('dm', Pins('G16 D16', dir = 'o')),
			Subsignal('odt', Pins('C13', dir = 'o')),
			Attrs(IO_TYPE = 'SSTL135_I', SLEWRATE = 'FAST')
		),
		Resource(
			'ddr3_pseudo_power', 0,
			# pseudo power pins, leave these at their default value
			Subsignal('vcc_virtual', PinsN('K16 D17 K15 K17 B18 C6', dir = 'o')),
			Subsignal('gnd_virtual', Pins('L15 L16', dir = 'o')),
			Attrs(IO_TYPE = 'SSTL135_II', SLEWRATE = 'FAST')
		),
		Resource(
			'adc', 0,
			Subsignal('ctrl', Pins('G1 F1', dir = 'o')),
			Subsignal('mux', Pins('F4 F3 F2 H1', dir = 'o')),
			Subsignal('sense', DiffPairs('H3', 'G3', dir = 'i'), Attrs(IO_TYPE = 'LVCMOS33D')),
			Attrs(IO_TYPE = 'LVCMOS33')
		),
		*SDCardResources(
			0,
			dat0 = 'J1', dat1 = 'K3', dat2 = 'L3', dat3 = 'M1', clk = 'K1', cmd = 'K2', cd = 'L1',
			attrs = Attrs(IO_TYPE = 'LVCMOS33', SLEWRATE = 'FAST')
		),
		DirectUSBResource(0, d_p = 'N1', d_n = 'M2', pullup = 'N2', attrs = Attrs(IO_TYPE = 'LVCMOS33'))
	]

	connectors = [
		Connector('io', 0, {
			'0': 'N17',
			'1': 'M18',
			'5': 'B10',
			'6': 'B9',
			'9': 'C8',
			'10': 'B8',
			'11': 'A8',
			'12': 'H2',
			'13': 'J2',
			'a0': 'L4',
			'a1': 'N3',
			'a2': 'N4',
			'a3': 'H4',
			'a4': 'G4',
			'cipo': 'N15',
			'copi': 'N16',
			'sck': 'R17',
			'scl': 'C9',
			'sda': 'C10'
		})
	]

	@property
	def required_tools(self) -> list[str]:
		return super().required_tools + [
			'dfu-suffix'
		]

	@property
	def command_templates(self) -> list[str]:
		return super().command_templates + [
			r'''{{invoke_tool('dfu-suffix')}} -v 1209 -p 5af0 -a {{name}}.bit'''
		]

	def toolchain_prepare(self, fragment: Fragment, name: str, **kwargs) -> BuildPlan:
		overrides = dict(ecppack_opts = '--compress --freq 38.8')
		overrides.update(kwargs)
		return super().toolchain_prepare(fragment, name, **overrides)

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		dfu_util = environ.get('DFU_UTIL', 'dfu-util')
		with products.extract(f'{name}.bit') as bitstream_filename:
			check_call([dfu_util, '-a', '0', '-D', bitstream_filename])


class OrangeCrabR0_2_25FPlatform(OrangeCrabR0_2Platform):
	pretty_name = 'OrangeCrab R0.2 25F'
	description = 'OrangeCrab R0.2 Lattice ECP5-25F Development Board'

class OrangeCrabR0_2_85FPlatform(OrangeCrabR0_2Platform):
	device = 'LFE5U-85F'

	pretty_name = 'OrangeCrab R0.2 85F'
	description = 'OrangeCrab R0.2 Lattice ECP5-85F Development Board'

if __name__ == '__main__':
	from ...test.blinky import Blinky
	OrangeCrabR0_2Platform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from .._util import _deprecated_import

__all__ = (
	'blackice_ii',
	'blackice',
	'colorlight_5a75b_r7_0',
	'ecp5_5g_evn',
	'ecpix5',
	'fomu_hacker',
	'fomu_pvt',
	'ice40_hx1k_blink_evn',
	'ice40_hx8k_b_evn',
	'ice40_up5k_b_evn',
	'icebreaker_bitsy',
	'icebreaker',
	'icestick',
	'icesugar',
	'icesugar_nano',
	'logicbone',
	'machxo3_sk',
	'nandlang_go',
	'orangecrab_r0_1',
	'orangecrab_r0_2',
	'supercon19badge',
	'tinyfpga_ax1',
	'tinyfpga_ax2',
	'tinyfpga_bx',
	'ulx3s',
	'upduino_v1',
	'upduino_v2',
	'upduino_v3',
	'versa_ecp5_5g',
	'versa_ecp5',
)

def __dir__() -> list[str]:
	return list({*globals(), *__all__})

def __getattr__(name: str):
	match name:
		case 'blackice_ii':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.ice40.{name}')
			from .ice40 import blackice_ii
			return blackice_ii
		case 'blackice':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.ice40.{name}')
			from .ice40 import blackice
			return blackice
		case 'colorlight_5a75b_r7_0':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.ecp5.{name}')
			from .ecp5 import colorlight_5a75b_r7_0
			return colorlight_5a75b_r7_0
		case 'ecp5_5g_evn':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.ecp5.{name}')
			from .ecp5 import ecp5_5g_evn
			return ecp5_5g_evn
		case 'ecpix5':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.ecp5.{name}')
			from .ecp5 import ecpix5
			return ecpix5
		case 'fomu_hacker':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.ice40.{name}')
			from .ice40 import fomu_hacker
			return fomu_hacker
		case 'fomu_pvt':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.ice40.{name}')
			from .ice40 import fomu_pvt
			return fomu_pvt
		case 'ice40_hx1k_blink_evn':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.ice40.{name}')
			from .ice40 import ice40_hx1k_blink_evn
			return ice40_hx1k_blink_evn
		case 'ice40_hx8k_b_evn':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.ice40.{name}')
			from .ice40 import ice40_hx8k_b_evn
			return ice40_hx8k_b_evn
		case 'ice40_up5k_b_evn':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.ice40.{name}')
			from .ice40 import ice40_up5k_b_evn
			return ice40_up5k_b_evn
		case 'icebreaker_bitsy':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.ice40.{name}')
			from .ice40 import icebreaker_bitsy
			return icebreaker_bitsy
		case 'icebreaker':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.ice40.{name}')
			from .ice40 import icebreaker
			return icebreaker
		case 'icestick':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.ice40.{name}')
			from .ice40 import icestick
			return icestick
		case 'icesugar':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.ice40.{name}')
			from .ice40 import icesugar
			return icesugar
		case 'icesugar_nano':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.ice40.{name}')
			from .ice40 import icesugar_nano
			return icesugar_nano
		case 'logicbone':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.ecp5.{name}')
			from .ecp5 import logicbone
			return logicbone
		case 'machxo3_sk':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.machxo3.{name}')
			from .machxo3 import machxo3_sk
			return machxo3_sk
		case 'nandlang_go':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.ice40.{name}')
			from .ice40 import nandland_go
			return nandland_go
		case 'orangecrab_r0_1':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.ecp5.{name}')
			from .ecp5 import orangecrab_r0_1
			return orangecrab_r0_1
		case 'orangecrab_r0_2':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.ecp5.{name}')
			from .ecp5 import orangecrab_r0_2
			return orangecrab_r0_2
		case 'supercon19badge':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.ecp5.{name}')
			from .ecp5 import supercon19badge
			return supercon19badge
		case 'tinyfpga_ax1':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.machxo2.{name}')
			from .machxo2 import tinyfpga_ax1
			return tinyfpga_ax1
		case 'tinyfpga_ax2':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.machxo2.{name}')
			from .machxo2 import tinyfpga_ax2
			return tinyfpga_ax2
		case 'tinyfpga_bx':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.ice40.{name}')
			from .ice40 import tinyfpga_bx
			return tinyfpga_bx
		case 'ulx3s':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.ecp5.{name}')
			from .ecp5 import ulx3s
			return ulx3s
		case 'upduino_v1':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.ice40.{name}')
			from .ice40 import upduino_v1
			return upduino_v1
		case 'upduino_v2':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.ice40.{name}')
			from .ice40 import upduino_v2
			return upduino_v2
		case 'upduino_v3':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.ice40.{name}')
			from .ice40 import upduino_v3
			return upduino_v3
		case 'versa_ecp5_5g':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.ecp5.{name}')
			from .ecp5 import versa_ecp5_5g
			return versa_ecp5_5g
		case 'versa_ecp5':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.ecp5.{name}')
			from .ecp5 import versa_ecp5
			return versa_ecp5
		case _:
			raise AttributeError(f'module {__name__!r} has no attribute {name!r}')



# SPDX-License-Identifier: BSD-2-Clause

from .blackice_ii          import BlackIceIIPlatform
from .blackice             import BlackIcePlatform
from .fomu_hacker          import FomuHackerPlatform
from .fomu_pvt             import FomuPVTPlatform
from .ice40_hx1k_blink_evn import ICE40HX1KBlinkEVNPlatform
from .ice40_hx8k_b_evn     import ICE40HX8KBEVNPlatform
from .ice40_up5k_b_evn     import ICE40UP5KBEVNPlatform
from .icebreaker_bitsy     import ICEBreakerBitsyPlatform
from .icebreaker           import ICEBreakerPlatform
from .icestick             import ICEStickPlatform
from .icesugar_nano        import ICESugarNanoPlatform
from .icesugar             import ICESugarPlatform
from .nandland_go          import NandlandGoPlatform
from .tinyfpga_bx          import TinyFPGABXPlatform
from .upduino_v1           import UpduinoV1Platform
from .upduino_v2           import UpduinoV2Platform
from .upduino_v3           import UpduinoV3Platform

__all__ = (
	'BlackIceIIPlatform',
	'BlackIcePlatform',
	'FomuHackerPlatform',
	'FomuPVTPlatform',
	'ICE40HX1KBlinkEVNPlatform',
	'ICE40HX8KBEVNPlatform',
	'ICE40UP5KBEVNPlatform',
	'ICEBreakerBitsyPlatform',
	'ICEBreakerPlatform',
	'ICEStickPlatform',
	'ICESugarNanoPlatform',
	'ICESugarPlatform',
	'NandlandGoPlatform',
	'TinyFPGABXPlatform',
	'UpduinoV1Platform',
	'UpduinoV2Platform',
	'UpduinoV3Platform',
)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                        import Attrs, Clock, Connector, DiffPairs, Pins, Resource, Subsignal
from torii.build.run                    import BuildPlan, BuildProducts
from torii.hdl.ir                       import Fragment
from torii.platform.resources.interface import DirectUSBResource
from torii.platform.resources.memory    import DDR3Resource, SDCardResources, SPIFlashResources
from torii.platform.resources.user      import ButtonResources, LEDResources
from torii.platform.vendor.lattice.ecp5 import ECP5Platform

__all__ = (
	'LogicbonePlatform',
	'Logicbone85FPlatform',
)

class LogicbonePlatform(ECP5Platform):
	name        = 'Logicbone'
	device      = 'LFE5UM5G-45F'
	package     = 'BG381'
	speed       = '8'
	default_clk = 'refclk'

	pretty_name = 'Logicbone'
	description = 'Logicbone Lattice ECP5-5G-45F Development Board'

	resources   = [
		Resource(
			'refclk', 0, Pins('M19', dir = 'i'), Clock(25e6), Attrs(IO_TYPE = 'LVCMOS18')
		),
		# SerDes connections.
		Resource(
			'serdes', 0,
			Subsignal('rx', DiffPairs('Y5', 'Y6')),
			Subsignal('tx', DiffPairs('W4', 'W5')),
		),
		Resource(
			'serdes', 1,
			Subsignal('rx', DiffPairs('Y8', 'Y7')),
			Subsignal('tx', DiffPairs('W8', 'W9')),
		),
		# VBUS Detection is on the USB-C PD controller for both ports.
		DirectUSBResource(
			'usb', 0,
			d_p = 'B12', d_n = 'C12', pullup = 'C16',
			attrs = Attrs(IO_TYPE = 'LVCMOS33')
		),
		DirectUSBResource(
			'usb', 1,
			d_p = 'R1',  d_n = 'T1',  pullup = 'T2',
			attrs = Attrs(IO_TYPE = 'LVCMOS33')
		),
		*LEDResources(pins = 'D16 C15 C13 B13', attrs = Attrs(IO_TYPE = 'LVCMOS33'), invert = True),
		*ButtonResources(pins = 'U2', attrs = Attrs(IO_TYPE = 'LVCMOS33'), invert = True),
		*SPIFlashResources(
			0,
			cs_n = 'R2', clk = 'U3', cipo = 'V2', copi = 'W2', wp_n = 'Y2', hold_n = 'W1',
			attrs = Attrs(IO_TYPE = 'LVCMOS33')
		),
		*SDCardResources(
			0,
			clk = 'E11', cmd = 'D15', cd = 'D14',
			dat0 = 'D13', dat1 = 'E13', dat2 = 'E15', dat3 = 'E13',
			attrs = Attrs(IO_TYPE = 'LVCMOS33')
		),
		Resource(
			'eth_clk125', 0, Pins('A19', dir = 'i'), Clock(125e6), Attrs(IO_TYPE = 'LVCMOS33')
		),
		# TODO(aki): Replace with `EthernetResource` when it supports `int` signal
		Resource(
			'eth_rgmii', 0,
			# Stolen for sys_reset usage on prototypes.
			# Subsignal('rst',     PinsN('U17', dir = 'o')),
			Subsignal('int', Pins('B20', dir = 'i')),
			Subsignal('mdc', Pins('D12', dir = 'o')),
			Subsignal('mdio', Pins('B19', dir = 'io')),
			Subsignal('tx_clk', Pins('A15', dir = 'o')),
			Subsignal('tx_ctl', Pins('B15', dir = 'o')),
			Subsignal('tx_data', Pins('A12 A13 C14 A14', dir = 'o')),
			Subsignal('rx_clk', Pins('B18', dir = 'i')),
			Subsignal('rx_ctl', Pins('A18', dir = 'i')),
			Subsignal('rx_data', Pins('B17 A17 B16 A16', dir = 'i')),
			Attrs(IO_TYPE = 'LVCMOS33')
		),
		DDR3Resource(
			0,
			rst_n = 'P1',
			clk_p = 'M4', clk_n = 'N5', clk_en = 'K4',
			cs_n = 'M3', we_n = 'E4', ras_n = 'L1', cas_n = 'M1',
			a = 'D5 F4 B3 F3 E5 C3 C4 A5 A3 B5 G3 F5 D2 A4 D3 E3', ba = 'B4 H5 N2',
			dqs_p = 'K2 H4', dqs_n = 'J1 G5',
			dq = 'G2 K1 F1 K3 H2 J3 G1 H1 B1 E1 A2 F2 C1 E2 C2 D1', dm = 'L4 J5', odt = 'C5',
			diff_attrs = Attrs(IO_TYPE = 'LVDS'),
			attrs = Attrs(IO_TYPE = 'SSTL135_I')
		)
	]
	connectors = [
		Connector(
			'P8', 0,
			' -   -   C20 D19 D20 E19 E20 F19 F20 G20 -   -   -   -   -   -   '
			' -   -   -   -   -   -   G19 H20 J20 K20 C18 D17 D18 E17 E18 F18 '
			' F17 G18 E16 F16 G16 H16 J17 J16 H18 H17 J19 K19 J18 K18 '
		),
		Connector(
			'P9', 0,
			'-   -   -   -   -   -   -   -   -   -   -   A11 B11 A10 C10 A9 '
			'B9  C11 A8  -   -   D9  C8  B8  A7  A6  B6  D8  C7  D7  C6  D6 '
			'-   -   -   -   -   -   -   -   -   B10 E10 -   -   - '
		),
	]

	def toolchain_prepare(self, fragment: Fragment, name: str, **kwargs) -> BuildPlan:
		overrides = dict(ecppack_opts = '--compress --spimode qspi --freq 38.8')
		overrides.update(kwargs)
		return super().toolchain_prepare(fragment, name, **overrides)

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call
		dfu_util = environ.get('DFU_UTIL', 'dfu-util')
		with products.extract(f'{name}.bit') as bitstream_filename:
			check_call([
				dfu_util, '-d', '1d50:615d', '-a', '0', '-R',
				'-D', bitstream_filename
			])


class Logicbone85FPlatform(LogicbonePlatform):
	name        = 'Logicbone (85F Variant)'
	device      = 'LFE5UM5G-85F'

if __name__ == '__main__':
	import argparse

	from ...test.blinky import Blinky

	parser = argparse.ArgumentParser()
	parser.add_argument(
		'--variant',
		choices = ('85', '45'),
		default = '45',
		help = 'platform variant (default: %(default)s)'
	)

	args = parser.parse_args()

	if args.variant == '45':
		platform = LogicbonePlatform()

	if args.variant == '85':
		platform = Logicbone85FPlatform()

	platform.build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                        import Attrs, Clock, Connector, DiffPairs, Pins, PinsN, Resource, Subsignal
from torii.build.run                    import BuildProducts
from torii.platform.resources.interface import UARTResource, ULPIResource
from torii.platform.resources.memory    import SPIFlashResources
from torii.platform.resources.user      import RGBLEDResource
from torii.platform.vendor.lattice.ecp5 import ECP5Platform

__all__ = (
	'ECPIX585Platform',
	'ECPIX545Platform',
)

class _ECPIX5Platform(ECP5Platform):
	package     = 'BG554'
	speed       = '8'
	default_clk = 'clk100'
	default_rst = 'rst'

	resources   = [
		Resource('rst', 0, PinsN('AB1', dir = 'i'), Attrs(IO_TYPE = 'LVCMOS33')),
		Resource(
			'clk100', 0, Pins('K23', dir = 'i'), Clock(100e6), Attrs(IO_TYPE = 'LVCMOS33')
		),
		RGBLEDResource(
			0,
			r = 'T23', g = 'R21', b = 'T22', invert = True,
			attrs = Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'UP')
		),
		RGBLEDResource(
			1,
			r = 'U21', g = 'W21', b = 'T24', invert = True,
			attrs = Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'UP')
		),
		RGBLEDResource(
			2,
			r = 'K21', g = 'K24', b = 'M21', invert = True,
			attrs = Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'UP')
		),
		RGBLEDResource(
			3,
			r = 'P21', g = 'R23', b = 'P22', invert = True,
			attrs = Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'UP')
		),
		UARTResource(
			0,
			rx = 'R26', tx = 'R24',
			attrs = Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'UP')
		),
		*SPIFlashResources(
			0,
			cs_n = 'AA2', clk = 'AE3', cipo = 'AE2', copi = 'AD2', wp_n = 'AF2', hold_n = 'AE1',
			attrs = Attrs(IO_TYPE = 'LVCMOS33')
		),
		# TODO(aki): Replace with `EthernetResource` when it gets an `rst` signal
		Resource(
			'eth_rgmii', 0,
			Subsignal('rst', PinsN('C13', dir = 'o')),
			Subsignal('mdio', Pins('A13', dir = 'io')),
			Subsignal('mdc', Pins('C11', dir = 'o')),
			Subsignal('tx_clk', Pins('A12', dir = 'o')),
			Subsignal('tx_ctrl', Pins('C9', dir = 'o')),
			Subsignal('tx_data', Pins('D8 C8 B8 A8', dir = 'o')),
			Subsignal('rx_clk', Pins('E11', dir = 'i')),
			Subsignal('rx_ctrl', Pins('A11', dir = 'i')),
			Subsignal('rx_data', Pins('B11 A10 B10 A9', dir = 'i')),
			Attrs(IO_TYPE = 'LVCMOS33')
		),
		Resource('eth_int', 0, PinsN('B13', dir = 'i'), Attrs(IO_TYPE = 'LVCMOS33')),
		# TODO(aki): Replace with `DDR3Resource` when it can have more fine-grained attrs
		Resource(
			'ddr3', 0,
			Subsignal('clk', DiffPairs('H3', 'J3', dir = 'o'), Attrs(IO_TYPE = 'SSTL15D_I')),
			Subsignal('clk_en', Pins('P1', dir = 'o')),
			Subsignal('we', PinsN('R3', dir = 'o')),
			Subsignal('ras', PinsN('T3', dir = 'o')),
			Subsignal('cas', PinsN('P2', dir = 'o')),
			Subsignal('a', Pins('T5 M3 L3 V6 K2 W6 K3 L1 H2 L2 N1 J1 M1 K1 H1', dir = 'o')),
			Subsignal('ba', Pins('U6 N3 N4', dir = 'o')),
			Subsignal(
				'dqs', DiffPairs('V4 V1', 'U5 U2', dir = 'io'),
				Attrs(IO_TYPE = 'SSTL15D_I', TERMINATION = 'OFF', DIFFRESISTOR = '100')
			),
			Subsignal(
				'dq', Pins('T4 W4 R4 W5 R6 P6 P5 P4 R1 W3 T2 V3 U3 W1 T1 W2', dir = 'io'),
				Attrs(TERMINATION = '75')
			),
			Subsignal('dm', Pins('U4 U1', dir = 'o')),
			Subsignal('odt', Pins('P3', dir = 'o')),
			Attrs(IO_TYPE = 'SSTL15_I', SLEWRATE = 'FAST')
		),
		Resource(
			'sata', 0,
			Subsignal('tx', DiffPairs('AD16', 'AD17', dir = 'o')),
			Subsignal('rx', DiffPairs('AF15', 'AF16', dir = 'i')),
			Attrs(IO_TYPE = 'LVDS')
		),
		ULPIResource(
			'ulpi', 0,
			data = 'M26 L25 L26 K25 K26 J23 P25 H25',
			clk = 'H24', clk_dir = 'i', dir = 'F22', nxt = 'F23',
			stp = 'H23', rst = 'E23', rst_invert = False,
			attrs = Attrs(IO_TYPE = 'LVCMOS33', SLEWRATE = 'SLOW')
		),
		Resource(
			'usbc_cfg', 0,
			Subsignal('scl', Pins('D24', dir = 'io')),
			Subsignal('sda', Pins('C24', dir = 'io')),
			Subsignal('dir', Pins('B23', dir = 'i')),
			Subsignal('id', Pins('D23', dir = 'i')),
			Subsignal('int', PinsN('B24', dir = 'i')),
			Attrs(IO_TYPE = 'LVCMOS33')
		),
		Resource(
			'usbc_mux', 0,
			Subsignal('en', Pins('C23', dir = 'oe')),
			Subsignal('amsel', Pins('B26', dir = 'oe')),
			Subsignal('pol', Pins('D26', dir = 'o')),
			Subsignal('lna', DiffPairs('AF9', 'AF10', dir = 'i'), Attrs(IO_TYPE = 'LVCMOS18D')),
			Subsignal('lnb', DiffPairs('AD10', 'AD11', dir = 'o'), Attrs(IO_TYPE = 'LVCMOS18D')),
			Subsignal('lnc', DiffPairs('AD7', 'AD8', dir = 'o'), Attrs(IO_TYPE = 'LVCMOS18D')),
			Subsignal('lnd', DiffPairs('AF6', 'AF7', dir = 'i'), Attrs(IO_TYPE = 'LVCMOS18D')),
			Attrs(IO_TYPE = 'LVCMOS33')
		),
	]

	connectors  = [
		Connector('pmod', 0, 'T25 U25 U24 V24 - - T26 U26 V26 W26 - -'),
		Connector('pmod', 1, 'U23 V23 U22 V21 - - W25 W24 W23 W22 - -'),
		Connector('pmod', 2, 'J24 H22 E21 D18 - - K22 J21 H21 D22 - -'),
		Connector('pmod', 3, 'E4 F4 E6 H4 - - F3 D4 D5 F5 - -'),
		Connector('pmod', 4, 'E26 D25 F26 F25 - - C26 C25 A25 A24 - -'),
		Connector('pmod', 5, 'D19 C21 B21 C22 - - D21 A21 A22 A23 - -'),
		Connector('pmod', 6, 'C16 B17 C18 B19 - - A17 A18 A19 C19 - -'),
		Connector('pmod', 7, 'D14 B14 E14 B16 - - C14 A14 A15 A16 - -'),
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		tool = environ.get('OPENFPGALOADER', 'openFPGALoader')
		with products.extract(f'{name}.bit') as bitstream_filename:
			check_call([
				tool, '-c', 'ft2232', '-m', bitstream_filename
			])

class ECPIX545Platform(_ECPIX5Platform):
	device      = 'LFE5UM5G-45F'

	pretty_name = 'ECPIX5 45F'
	description = 'Lambda Concept ECPIX5 45F Lattice ECP5-5G-45F Development Board'

	resources   = [
		*_ECPIX5Platform.resources,
		# The IT6613E HDMI transmitter has access to 8 bits per color channel.
		Resource(
			'it6613e', 0,
			Subsignal('rst', PinsN('N6', dir = 'o')),
			Subsignal('scl', Pins('C17', dir = 'io')),
			Subsignal('sda', Pins('E17', dir = 'io')),
			Subsignal('pclk', Pins('C1', dir = 'o')),
			Subsignal('vsync', Pins('A4', dir = 'o')),
			Subsignal('hsync', Pins('B4', dir = 'o')),
			Subsignal('de', Pins('A3', dir = 'o')),
			Subsignal(
				'd',
				Subsignal('b', Pins('B3 C3 D3 B1 C2 D2 D1 E3', dir = 'o')),
				Subsignal('g', Pins('E1 F2 F1 D17 D16 E16 J6 H6', dir = 'o')),
				Subsignal('r', Pins('E10 D11 D10 C10 D9 E8 H5 J4', dir = 'o')),
			),
			Subsignal('mclk', Pins('E19', dir = 'o')),
			Subsignal('sck', Pins('D6', dir = 'o')),
			Subsignal('ws', Pins('C6', dir = 'o')),
			Subsignal('i2s', Pins('A6 B6 A5 C5', dir = 'o')),
			Subsignal('int', PinsN('C4', dir = 'i')),
			Attrs(IO_TYPE = 'LVTTL33')
		),
	]


class ECPIX585Platform(_ECPIX5Platform):
	device      = 'LFE5UM5G-85F'

	pretty_name = 'ECPIX5 85F'
	description = 'Lambda Concept ECPIX5 85F Lattice ECP5-5G-85F Development Board'

	resources   = [
		*_ECPIX5Platform.resources,
		# The IT6613E HDMI transmitter has access to 12 bits per color channel. The LFE5UM5G-85F
		# has an additional I/O bank which is used to provide the lower 4 bits of each channel.
		Resource(
			'it6613e', 0,
			Subsignal('rst', PinsN('N6', dir = 'o')),
			Subsignal('scl', Pins('C17', dir = 'io')),
			Subsignal('sda', Pins('E17', dir = 'io')),
			Subsignal('pclk', Pins('C1', dir = 'o')),
			Subsignal('vsync', Pins('A4', dir = 'o')),
			Subsignal('hsync', Pins('B4', dir = 'o')),
			Subsignal('de', Pins('A3', dir = 'o')),
			Subsignal(
				'd',
				Subsignal('b', Pins('AD25 AC26 AB24 AB25 B3 C3 D3 B1 C2 D2 D1 E3', dir = 'o')),
				Subsignal('g', Pins('AA23 AA22 AA24 AA25 E1 F2 F1 D17 D16 E16 J6 H6', dir = 'o')),
				Subsignal('r', Pins('AD26 AE25 AF25 AE26 E10 D11 D10 C10 D9 E8  H5 J4', dir = 'o')),
			),
			Subsignal('mclk', Pins('E19', dir = 'o')),
			Subsignal('sck', Pins('D6', dir = 'o')),
			Subsignal('ws', Pins('C6', dir = 'o')),
			Subsignal('i2s', Pins('A6 B6 A5 C5', dir = 'o')),
			Subsignal('int', PinsN('C4', dir = 'i')),
			Attrs(IO_TYPE = 'LVTTL33')
		),
	]


if __name__ == '__main__':
	import argparse

	from ...test.blinky import Blinky

	parser = argparse.ArgumentParser()
	parser.add_argument(
		'--variant',
		choices = ('85', '45'),
		default = '85',
		help = 'platform variant (default: %(default)s)'
	)

	args = parser.parse_args()
	if args.variant == '85':
		platform = ECPIX585Platform()
	else:
		platform = ECPIX545Platform()

	platform.build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

import subprocess

from torii.build                        import Attrs, Clock, Connector, Pins, Resource
from torii.platform.resources.interface import UARTResource
from torii.platform.resources.memory    import SPIFlashResources, SRAMResource
from torii.platform.resources.user      import ButtonResources, LEDResources, RGBLEDResource
from torii.platform.vendor.xilinx       import XilinxPlatform

'''
Example Usage:
	platform = CModA7_35Platform(toolchain = 'Symbiflow')
	platform.build(Top(), do_program = True)

Supported programmer:
	openFPGALoader
'''

__all__ = (
	'CmodA7_15Platform',
	'CmodA7_35Platform',
)

class _CmodA7Platform(XilinxPlatform):
	package     = 'cpg236'
	speed       = '1'
	default_clk = 'clk12'
	resources   = [
		Resource(
			'clk12', 0, Pins('L17', dir = 'i'), Clock(12e6), Attrs(IOSTANDARD = 'LVCMOS33')
		),
		*LEDResources(pins = 'A17 C16', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),
		RGBLEDResource(
			0,
			r = 'C17', g = 'B16', b = 'B17', invert = True,
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		*ButtonResources(pins = 'A18 B18', attrs = Attrs(IOSTANDARD='LVCMOS33')),
		UARTResource(0, rx = 'J17', tx = 'J18', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),
		*SPIFlashResources(
			0,
			cs_n = 'K19', clk = 'E19', copi = 'D19', cipo = 'D18', wp_n = 'G18',
			hold_n = 'F18',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		SRAMResource(
			0,
			cs_n = 'N19', oe_n = 'P19', we_n = 'R19',
			a = 'M18 M19 K17 N17 P17 P18 R18 W19 U19 V19 W18 T17 T18 U17 U18 V16 W16 W17 V15',
			d = 'W15 W13 W14 U15 U16 V13 V14 U14'
		),
		# One-wire interface to crypto authentication device
		# May not be populated on the board
		Resource('atsha204a', 0, Pins('D17', dir = 'io'), Attrs(IOSTANDARD = 'LVCMOS33'))
	]
	connectors  = [
		Connector('pmod', 0, 'G17 G19 N18 L18 - - H17 H19 J19 K18 - -'), # JA
		# Pin 24/25 are VCC and GND
		# Pin 15/16 are analog (XADC)
		Connector(
			'gpio', 0,
			'M3  L3 A16 K3  C15 H1  A15 B15 A14 J3  J1  K2 '
			'L1  L2 -   -   M1  N3  P3  M2  N1  N2  P1   - '
			'-   R3 T3  R2  T1  T2  U1  W2  V2  W3  V3  W5 '
			'V4  U4 V5  W4  U5  U2  W6  U3  U7  W7  U8  V8 '
		),
		Connector('xadc', 0, {
			'vaux4_n': 'G2',
			'vaux4_p': 'G3',
			'vaux12_n': 'J2',
			'vaux12_p': 'H',
		})
	]

	def toolchain_program(self, products, name):
		with products.extract(f'{name}.bit') as bitstream_filename:
			subprocess.check_call([
				'openFPGALoader', '-c', 'digilent', '--fpga-part', self.device, bitstream_filename
			])


class CmodA7_15Platform(_CmodA7Platform):
	device      = 'xc7a15t'

	pretty_name = 'Cmod A7-15T'
	description = 'Digilent Cmod A7-15T Xilinx Artix7-15T FPGA Module'

class CmodA7_35Platform(_CmodA7Platform):
	device      = 'xc7a35t'

	pretty_name = 'Cmod A7-35T'
	description = 'Digilent Cmod A7-15T Xilinx Artix7-35T FPGA Module'

if __name__ == '__main__':
	from ...test.blinky import Blinky
	CmodA7_35Platform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                        import Attrs, Connector, Pins, Resource, Subsignal
from torii.build.run                    import BuildProducts
from torii.platform.resources.interface import I2CResource, UARTResource
from torii.platform.resources.memory    import SDCardResources
from torii.platform.resources.user      import LEDResources
from torii.platform.vendor.altera       import AlteraPlatform

__all__ = (
	'Chameleon96Platform',
)

class Chameleon96Platform(AlteraPlatform):
	device      = '5CSEBA6' # Cyclone V SE 110K LEs
	package     = 'U19'     # UBGA-484
	speed       = 'I7'
	default_clk = 'cyclonev_oscillator'

	pretty_name = 'Chameleon96'
	description = 'Altera Cyclone V SoC Development Board'

	resources   = [
		# WIFI and BT LEDs
		*LEDResources(
			pins = 'Y19 Y20',
			attrs = Attrs(io_standard = '2.5 V')
		),
		# TDA19988 HDMI transmitter
		Resource(
			'tda19988', 0,
			Subsignal('vpa', Pins('W8 W7 V6 V5 U6', dir = 'o')), # bits 3 to 7
			Subsignal('vpb', Pins('AB5 AA5 AA8 AB8 AB9 Y11', dir = 'o')), # bits 2 to 7
			Subsignal('vpc', Pins('W6 Y5 AB7 AA7 AA6', dir = 'o')), # bits 3 to 7
			Subsignal('pclk', Pins('AB10', dir = 'o')),
			Subsignal('hsync', Pins('V10', dir = 'o')),
			Subsignal('vsync', Pins('V7', dir = 'o')),
			Subsignal('de', Pins('Y8', dir = 'o')),
			Attrs(io_standard = '1.8 V')
		),
		I2CResource(
			'tda19988_i2c', 0,
			scl = 'U7', sda = 'U10',
			attrs = Attrs(io_standard = '1.8 V'),
		),
		Resource(
			'tda19988_i2s', 0,
			Subsignal('mclk', Pins('V11', dir = 'o')), # OSC_IN/AP3
			Subsignal('txd', Pins('AA11', dir = 'o')), # AP1
			Subsignal('txc', Pins('W11', dir = 'o')), # ACLK
			Subsignal('txfs', Pins('V9', dir = 'o')), # AP0
			Attrs(io_standard = '1.8 V')
		),
		# Wifi and BT module
		*SDCardResources(
			'wifi', 0,
			clk = 'AB20', cmd = 'AB18', dat0 = 'Y14', dat1 = 'AB19', dat2 = 'AA18', dat3 = 'AB15',
			attrs = Attrs(io_standard = '1.8 V'),
		),
		Resource(
			'bt', 0,
			Subsignal('host_wake', Pins('AB12', dir = 'o')),
			Attrs(io_standard = '1.8 V'),
		),
		Resource(
			'bt_i2s', 0,
			Subsignal('txd', Pins('Y15', dir = 'o')), # BT_PCM_IN
			Subsignal('rxd', Pins('Y16', dir = 'i')), # BT_PCM_OUT
			Subsignal('txc', Pins('AA13', dir = 'i')), # BT_PCM_CLK
			Subsignal('txfs', Pins('AB13', dir = 'i')), # BT_PCM_SYNC
			Attrs(io_standard = '1.8 V'),
		),
		UARTResource(
			'bt_uart', 0,
			rx = 'AB14', cts = 'AB17', tx = 'AA15', rts = 'AA16', role = 'dte',
			attrs = Attrs(io_standard = '1.8 V'),
		),
	]

	connectors  = [
		# J3, 2x20 expansion port
		Connector(
			'J', 3,
			'-   -   Y13 -   W14 -   C5  -   C6  -  '
			'-   -   -   -   -   E5  -   F5  -   A6 '
			'-   A5  -   -   -   -   -   -   -   -  '
			'-   -   -   -   -   -   -   -   -   -  '
		),
		# J8, 2x30 high speed expansion port (MIPI CSI)
		Connector(
			'J', 8,
			'-   V16 -   U17 -   -   -   V17 -   W18 '
			'-   -   -   U18 W12 V19 -   -   -   -   '
			'-   -   -   -   -   -   -   -   -   -   '
			'-   -   -   -   -   -   -   -   -   -   '
			'-   -   -   -   -   -   -   -   -   -   '
			'-   -   -   -   -   -   -   -   -   -   '
		),
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		quartus_pgm = environ.get('QUARTUS_PGM', 'quartus_pgm')
		with products.extract(f'{name}.sof') as bitstream_filename:
			# The @2 selects the second device in the JTAG chain, because this chip
			# puts the ARM cores first.
			check_call([
				quartus_pgm, '--haltcc', '--mode', 'JTAG',
				'--operation', 'P;' + bitstream_filename + '@2'
			])


if __name__ == '__main__':
	from ...test.blinky import Blinky
	Chameleon96Platform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                         import Attrs, Clock, Connector, Pins, Resource
from torii.build.run                     import BuildProducts
from torii.platform.resources.interface  import UARTResource
from torii.platform.resources.memory     import SRAMResource
from torii.platform.resources.user       import ButtonResources, LEDResources, SwitchResources
from torii.platform.vendor.lattice.ice40 import ICE40Platform

__all__ = (
	'BlackIceIIPlatform',
)

class BlackIceIIPlatform(ICE40Platform):
	device      = 'iCE40HX4K'
	package     = 'TQ144'
	default_clk = 'clk100'

	pretty_name = 'BlackIce II'
	description = 'myStorm BlackIce II Lattice iCE40-HX4k Development Board'

	resources   = [
		Resource(
			'clk100', 0,
			Pins('129', dir = 'i'),
			Clock(100e6), Attrs(GLOBAL = True, IO_STANDARD = 'SB_LVCMOS')
		),
		*LEDResources(pins = '71 67 68 70', attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')),
		# Color aliases
		Resource('led_b', 0, Pins('71', dir = 'o'), Attrs(IO_STANDARD = 'SB_LVCMOS')),
		Resource('led_g', 0, Pins('67', dir = 'o'), Attrs(IO_STANDARD = 'SB_LVCMOS')),
		Resource('led_o', 0, Pins('68', dir = 'o'), Attrs(IO_STANDARD = 'SB_LVCMOS')),
		Resource('led_r', 0, Pins('70', dir = 'o'), Attrs(IO_STANDARD = 'SB_LVCMOS')),
		*ButtonResources(pins = '63 64', invert = True, attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')),
		*SwitchResources(pins = '37 38 39 41', invert = True, attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')),
		UARTResource(
			0,
			rx = '88', tx = '85', rts = '91', cts = '94',
			attrs = Attrs(IO_STANDARD = 'SB_LVCMOS', PULLUP = 1),
			role = 'dce'
		),
		SRAMResource(
			0,
			cs_n = '23', oe_n = '29', we_n = '120',
			a = '137 138 139 141 142 42 43 44 73 74 75 76 115 116 117 118 119 78',
			d = '136 135 134 130 125 124 122 121 62 61 60 56 55 48 47 45',
			dm_n = '24 28',
			attrs = Attrs(IO_STANDARD = 'SB_LVCMOS'),
		),
	]
	connectors  = [
		Connector('pmod', 0, '94 91 88 85 - - 95 93 90  87 - -'),  # PMOD1/2
		Connector('pmod', 1, '105 102 99 97 - - 104 101 98 96 - -'),  # PMOD3/4
		Connector('pmod', 2, '143 114 112 107 - - 144 113 110 106 - -'),  # PMOD5/6
		Connector('pmod', 3, '10 9 2 1 - - 8 7 4 3 - -'),  # PMOD7/8
		Connector('pmod', 4, '20 19 16 15 - - 18 17 12 11 - -'),  # PMOD9/10
		Connector('pmod', 5, '34 33 22 21 - - 32 31 26 25 - -'),  # PMOD11/12
		Connector('pmod', 6, '37 38 39 41 - -'),  # PMOD13
		Connector('pmod', 7, '71 67 68 70 - -'),  # PMOD14
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from subprocess import check_call

		with products.extract(f'{name}.bin') as bitstream_filename:
			check_call([
				'cp', bitstream_filename, '/dev/ttyACM0'
			])


if __name__ == '__main__':
	from ...test.blinky import Blinky
	BlackIceIIPlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                         import Attrs, Clock, Connector, Pins, PinsN, Resource
from torii.build.run                     import BuildProducts
from torii.platform.resources.interface  import DirectUSBResource, UARTResource
from torii.platform.resources.memory     import SPIFlashResources
from torii.platform.resources.user       import LEDResources, SwitchResources
from torii.platform.vendor.lattice.ice40 import ICE40Platform

__all__ = (
	'ICESugarPlatform',
)

class ICESugarPlatform(ICE40Platform):
	device      = 'iCE40UP5K'
	package     = 'SG48'
	default_clk = 'clk12'

	pretty_name = 'iCESugar'
	description = 'iCESugar Lattice iCE40-UP5K Development Board'

	resources   = [
		Resource(
			'clk12', 0, Pins('35', dir = 'i'), Clock(12e6),
			Attrs(GLOBAL = True, IO_STANDARD = 'LVCMOS33')
		),
		*LEDResources(pins = '40 41 39', invert = True, attrs = Attrs(IO_STANDARD = 'LVCMOS33')),
		# Semantic aliases
		Resource('led_r', 0, PinsN('40', dir = 'o'), Attrs(IO_STANDARD = 'LVCMOS33')),
		Resource('led_g', 0, PinsN('41', dir = 'o'), Attrs(IO_STANDARD = 'LVCMOS33')),
		Resource('led_b', 0, PinsN('39', dir = 'o'), Attrs(IO_STANDARD = 'LVCMOS33')),
		*SwitchResources(
			pins = '18 19 20 21',
			attrs = Attrs(IO_STANDARD = 'LVCMOS33')
		),
		UARTResource(
			0,
			rx = '4', tx = '6',
			attrs = Attrs(IO_STANDARD = 'LVTTL33', PULLUP = 1)
		),
		*SPIFlashResources(
			0,
			cs_n = '16', clk = '15', copi = '14', cipo = '17', wp_n = '12', hold_n = '13',
			attrs = Attrs(IO_STANDARD = 'LVCMOS33')
		),
		DirectUSBResource(
			0,
			d_p = '10', d_n = '9', pullup = '11', attrs = Attrs(IO_STANDARD = 'LVCMOS33')
		),
	]

	connectors = [
		Connector('pmod', 0, '10 6 3 48 - -  9 4 2 47 - -'), # PMOD1 - IO pins shared by USB
		Connector('pmod', 1, '46 44 42 37 - - 45 43 38 36 - -'), # PMOD2
		Connector('pmod', 2, '34 31 27 25 - - 32 28 26 23 - -'), # PMOD3
		Connector('pmod', 3, '21 20 19 18 - - - - - - - -'), # PMOD4 - IO pins used for switches via jumpers
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		iceprog = environ.get('ICEPROG', 'iceprog')
		with products.extract(f'{name}.bin') as bitstream_filename:
			check_call([iceprog, bitstream_filename])


if __name__ == '__main__':
	from ...test.blinky import Blinky
	ICESugarPlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                        import Attrs, Clock, Connector, Pins, Resource, Subsignal
from torii.build.run                    import BuildProducts
from torii.platform.resources.interface import SPIResource, UARTResource
from torii.platform.resources.user      import ButtonResources, LEDResources, SwitchResources
from torii.platform.vendor.altera       import AlteraPlatform

__all__ = (
	'DE10NanoPlatform',
)

# The MiSTer platform is built around the DE10-Nano; if you update one you should update the other.
class DE10NanoPlatform(AlteraPlatform):
	device      = '5CSEBA6' # Cyclone V 110K LEs
	package     = 'U23'     # UBGA-484
	speed       = 'I7'
	default_clk = 'clk50'

	pretty_name = 'DE10-Nano'
	description = 'terasIC DE10-Nano Altera Cyclone V Development Board'

	resources   = [
		Resource(
			'clk50', 0, Pins('V11', dir = 'i'), Clock(50e6), Attrs(io_standard = '3.3-V LVTTL')
		),
		Resource(
			'clk50', 1, Pins('Y13', dir = 'i'), Clock(50e6), Attrs(io_standard = '3.3-V LVTTL')
		),
		Resource(
			'clk50', 2, Pins('E11', dir = 'i'), Clock(50e6), Attrs(io_standard = '3.3-V LVTTL')
		),
		*LEDResources(
			pins = 'W15 AA24 V16 V15 AF26 AE26 Y16 AA23',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		*ButtonResources(
			pins = 'AH17 AH16', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		*SwitchResources(
			pins = 'Y24 W24 W21 W20',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		# Arduino header
		UARTResource(
			0,
			rx = 'AG13', tx = 'AF13',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		# LTC2308 analogue-to-digital converter
		SPIResource(
			0,
			cs_n = 'U9', clk = 'V10', copi = 'AC4', cipo = 'AD4',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		# ADV7513 HDMI transmitter
		# Note this has a lot of input formats for tx_d, but this defaults to RGB24
		Resource(
			'adv7513', 0,
			Subsignal('tx_d_r', Pins('AD12 AE12 W8 Y8 AD11 AD10 AE11 Y5', dir = 'o')),
			Subsignal('tx_d_g', Pins('AF10 Y4 AE9 AB4 AE7 AF6 AF8 AF5', dir = 'o')),
			Subsignal('tx_d_b', Pins('AE4 AH2 AH4 AH5 AH6 AG6 AF9 AE8', dir = 'o')),
			Subsignal('tx_clk', Pins('AG5', dir = 'o')),
			Subsignal('tx_de', Pins('AD19', dir = 'o')),
			Subsignal('tx_hs', Pins('T8', dir = 'o')),
			Subsignal('tx_vs', Pins('V13', dir = 'o')),
			Subsignal('tx_int', Pins('AF11', dir = 'i')),
			Subsignal('i2s0', Pins('T13', dir = 'o')),
			Subsignal('mclk', Pins('U11', dir = 'o')),
			Subsignal('lrclk', Pins('T11', dir = 'o')),
			Subsignal('sclk', Pins('T12', dir = 'o')),
			Subsignal('scl', Pins('U10', dir = 'io')),
			Subsignal('sda', Pins('AA4', dir = 'io')),
			Attrs(io_standard = '3.3-V LVTTL')
		),
	]
	connectors  = [
		# Located on the top of the board, above the chip.
		Connector(
			'gpio', 0,
			'V12  E8   W12  D11  D8   AH13 AF7  AH14 AF4  AH3  '
			'-    -    AD5  AG14 AE23 AE6  AD23 AE24 D12  AD20 '
			'C12  AD17 AC23 AC22 Y19  AB23 AA19 W11  -    -    '
			'AA18 W14  Y18  Y17  AB25 AB26 Y11  AA26 AA13 AA11 '
		),
		# Located on the bottom of the board.
		Connector(
			'gpio', 1,
			'Y15  AC24 AA15 AD26 AG28 AF28 AE25 AF27 AG26 AH27 '
			'-    -    AG25 AH26 AH24 AF25 AG23 AF23 AG24 AH22 '
			'AH21 AG21 AH23 AA20 AF22 AE22 AG20 AF21 -    -    '
			'AG19 AH19 AG18 AH18 AF18 AF20 AG15 AE20 AE19 AE17 '
		),
		Connector(
			'arduino', 0,
			'AG13 AF13 AG10 AG9  U14  U13  AG8  AH8  '
			'AF17 AE15 AF15 AG16 AH11 AH12 AH9  AG11 '
			'AH7  -    -    -    -    -    -    -    '
		),
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		quartus_pgm = environ.get('QUARTUS_PGM', 'quartus_pgm')
		with products.extract(f'{name}.sof') as bitstream_filename:
			# The @2 selects the second device in the JTAG chain, because this chip
			# puts the ARM cores first.
			check_call([
				quartus_pgm, '--haltcc', '--mode', 'JTAG',
				'--operation', 'P;' + bitstream_filename + '@2'
			])


if __name__ == '__main__':
	from ...test.blinky import Blinky
	DE10NanoPlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                        import Attrs, Clock, Connector, DiffPairs, Pins, PinsN, Resource, Subsignal
from torii.platform.resources.display   import VGADACResource
from torii.platform.resources.interface import I2CResource, UARTResource
from torii.platform.resources.memory    import SDCardResources
from torii.platform.resources.user      import LEDResources
from torii.platform.vendor.xilinx       import XilinxPlatform

__all__ = (
	'Mega65r3Platform',
)

class Mega65r3Platform(XilinxPlatform):
	device = 'xc7a200t'
	package = 'fbg484'
	speed = '2'
	default_clk = 'clk100'

	pretty_name = 'Mega65'
	description = 'Mega65 Xilinx Artix7-200T Based 8-bit computer'

	resources = [
		Resource('clk100', 0, Pins('V13', dir = 'i'), Clock(100e6), Attrs(IOSTANDARD = 'LVCMOS33')),
		*LEDResources(pins = 'U22', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),
		Resource(
			'iec', 0,
			Subsignal('rst', Pins('AB21', dir = 'o')),
			Subsignal('atn', Pins('N17', dir = 'o')),
			Subsignal('data_en', Pins('Y21', dir = 'o')),
			Subsignal('data_o', Pins('Y22', dir = 'o')),
			Subsignal('data_i', Pins('AB22', dir = 'i'), Attrs(PULLUP = 'TRUE')),
			Subsignal('clk_en', Pins('AA21', dir = 'o')),
			Subsignal('clk_o', Pins('Y19', dir = 'o')),
			Subsignal('clk_i', Pins('Y18', dir = 'i'), Attrs(PULLUP = 'TRUE')),
			Subsignal('srq_en', Pins('AB20', dir = 'o')),
			Subsignal('srq_o', Pins('U20', dir = 'o')),
			Subsignal('srq_i', Pins('AA18', dir = 'i')),
			Attrs(IOSTANDARD = 'LVCMOS33'),
		),
		Resource(
			'cart', 0,
			Subsignal('ctrl_en', Pins('G18', dir = 'o')),
			Subsignal('ctrl_dir', Pins('U17', dir = 'o')),
			Subsignal('addr_en', Pins('L19', dir = 'o')),
			Subsignal('addr_dir_lo', Pins('L21', dir = 'o')),
			Subsignal('addr_dir_hi', Pins('L18', dir = 'o')),
			Subsignal('data_en', Pins('U21', dir = 'o')),
			Subsignal('data_dir', Pins('V22', dir = 'o')),
			Subsignal('phi2', Pins('V17', dir = 'o')),
			Subsignal('dotclk', Pins('AA19', dir = 'o')),
			Subsignal('rst', Pins('N14', dir = 'o')),
			Subsignal('nmi', Pins('W17', dir = 'i')),
			Subsignal('irq', Pins('P14', dir = 'i')),
			Subsignal('dma', Pins('P15', dir = 'i')),
			Subsignal('exrom', Pins('R19', dir = 'io')),
			Subsignal('ba', Pins('N13', dir = 'io')),
			Subsignal('rw', Pins('R18', dir = 'io')),
			Subsignal('rom_lo', Pins('AB18', dir = 'io')),
			Subsignal('rom_hi', Pins('T18', dir = 'io')),
			Subsignal('io1', Pins('N15', dir = 'io')),
			Subsignal('game', Pins('W22', dir = 'io')),
			Subsignal('io2', Pins('AA20', dir = 'io')),
			Subsignal('data', Pins('P16 R17 P20 R16 U18 V18 W20 W21', dir = 'io')),
			Subsignal('addr', Pins(
				'K19 K18 K21 M22 L20 J20 J21 K22 '
				'H17 H20 G20 J15 H19 M20 N22 H18',
				dir = 'io'
			)),
			Attrs(IOSTANDARD = 'LVCMOS33'),
		),
		Resource(
			'keyboard', 0,
			Subsignal('clk', Pins('A14', dir = 'o')),
			Subsignal('out', Pins('A13', dir = 'o')),
			Subsignal('in', Pins('C13', dir = 'i')),
			Subsignal('tck', Pins('E13', dir = 'o')),
			Subsignal('tdo', Pins('E14', dir = 'o')),
			Subsignal('tdi', Pins('D15', dir = 'i')),
			Subsignal('tms', Pins('D14', dir = 'o')),
			Subsignal('jtag_en', Pins('B13', dir = 'o')),
			Attrs(IOSTANDARD = 'LVCMOS33'),
		),
		Resource(
			'paddle', 0,
			Subsignal('in', Pins('H13 G15 J14 J22', dir = 'i')),
			Subsignal('drain', Pins('H22', dir = 'o')),
			Attrs(IOSTANDARD = 'LVCMOS33'),
		),
		Resource(
			'joystick', 0,
			Subsignal('up', Pins('C14', dir = 'i')),
			Subsignal('down', Pins('F16', dir = 'i')),
			Subsignal('left', Pins('F14', dir = 'i')),
			Subsignal('right', Pins('F13', dir = 'i')),
			Subsignal('fire', Pins('E17', dir = 'i')),
			Attrs(IOSTANDARD = 'LVCMOS33'),
		),
		Resource(
			'joystick', 1,
			Subsignal('up', Pins('W19', dir = 'i')),
			Subsignal('down', Pins('P17', dir = 'i')),
			Subsignal('left', Pins('F21', dir = 'i')),
			Subsignal('right', Pins('C15', dir = 'i')),
			Subsignal('fire', Pins('F15', dir = 'i')),
			Attrs(IOSTANDARD = 'LVCMOS33'),
		),
		VGADACResource(
			0,
			clk = 'AA9',
			r = 'U15 V15 T14 Y17 Y16 AB17 AA16 AB16',
			g = 'Y14 W14 AA15 AB15 Y13 AA14 AA13 AB13',
			b = 'W10 Y12 AB12 AA11 AB11 Y11 AB10 AA10',
			hs = 'W12', vs = 'V14',
			extras = [
				Subsignal('sync', PinsN('V10', dir = 'o')),
				Subsignal('blank', PinsN('W11', dir = 'o')),
			],
			attrs = Attrs(IOSTANDARD = 'LVCMOS33'),
		),
		# TODO(aki): Replace with `HDMIResource` when merged + released in Torii
		Resource(
			'hdmi', 0,
			Subsignal('clk', DiffPairs('W1', 'Y1', dir = 'o')),
			Subsignal('data', DiffPairs('AA1 AB3 AA5', 'AB1 AB2 AB5', dir = 'o')),
			Subsignal('scl', Pins('AB7', dir = 'io')),
			Subsignal('sda', Pins('V9', dir = 'io')),
			Subsignal('en', Pins('AB8', dir = 'o')), # aka ls_oe
			Subsignal('hpd', Pins('Y8', dir = 'i')),
			Subsignal('hpd_en', Pins('M15', dir = 'o')),
			Subsignal('cec', Pins('W9', dir = 'o')),
			Attrs(IOSTANDARD = 'LVCMOS33'),
		),
		I2CResource(0, scl = 'A15', sda = 'A16', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),
		# Seeed Grove
		I2CResource('grove', 0, scl = 'G21', sda = 'G22', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),
		Resource(
			'audio', 0,
			Subsignal('left', Pins('L6', dir = 'o')),
			Subsignal('right', Pins('F4', dir = 'o')),
			Subsignal('sd', Pins('F18', dir = 'o')),
			Subsignal('speaker', Pins('E16', dir = 'o')),
			Subsignal('mclk', Pins('D16', dir = 'o')),
			Subsignal('bclk', Pins('E19', dir = 'o')),
			Subsignal('sync', Pins('F19', dir = 'o')),
			Attrs(IOSTANDARD = 'LVCMOS33'),
		),
		# TODO(aki): Replace with `SPIFlashResource` when more granular attrs are possible
		Resource(
			'spi_flash', 0,
			Subsignal('data', Pins('P22 R22 P21 R21', dir = 'io'), Attrs(PULLUP = 'TRUE')),
			Subsignal('cs', PinsN('T19', dir = 'o')),
		),
		# TODO(aki): Replace with `HyperBusResource` when more granular attrs are possible
		Resource(
			'hyper_ram', 0,
			Subsignal('clk', Pins('D22', dir = 'o'), Attrs(SLEW = 'FAST', DRIVE = '16')),
			Subsignal('data', Pins('A21 D21 C20 A20 B20 A19 E21 E22', dir = 'io'), Attrs(SLEW = 'FAST', DRIVE = '16')),
			Subsignal('rwds', Pins('B21', dir = 'io'), Attrs(SLEW = 'FAST', DRIVE = '16')),
			Subsignal('rst', Pins('B22', dir = 'o')),
			Subsignal('cs', Pins('C22', dir = 'o')),
			Attrs(IOSTANDARD = 'LVCMOS33', PULLUP = 'FALSE'),
		),
		# TODO(aki): Replace with `EthernetResource` when more granular attrs are possible
		Resource(
			'ethernet', 0,
			Subsignal('led', Pins('R14', dir = 'o')),
			Subsignal('clk', Pins('L4', dir = 'o'), Attrs(SLEW = 'FAST')),
			Subsignal('rst', Pins('K6', dir = 'o')),
			Subsignal('mdio', Pins('L5', dir = 'io')),
			Subsignal('mdc', Pins('J6', dir = 'o')),
			Subsignal('rxd', Pins('P4 L1', dir = 'i')),
			Subsignal('txd', Pins('L3 K3', dir = 'o'), Attrs(SLEW = 'SLOW', DRIVE = '4')),
			Subsignal('rxer', Pins('M6', dir = 'i')),
			Subsignal('txen', Pins('J4', dir = 'o'), Attrs(SLEW = 'SLOW', DRIVE = '4')),
			Subsignal('rxdv', Pins('K4', dir = 'i')),
			Attrs(IOSTANDARD = 'LVCMOS33'),
		),
		UARTResource(0, rx = 'L14', tx = 'L13', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),
		Resource(
			'max10', 0,
			Subsignal('tx', Pins('M13', dir = 'i')),
			Subsignal('rx', Pins('K16', dir = 'o')),
			Subsignal('rst', Pins('L16', dir = 'io')),
			Attrs(IOSTANDARD = 'LVCMOS33'),
		),
		*SDCardResources(
			0,
			clk = 'B17', cmd = 'B16', cd = 'D17', wp_n = 'C17',
			dat0 = 'B18', dat1 = 'C18', dat2 = 'C19', dat3 = 'B15',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33'),
		),
		*SDCardResources(
			1,
			clk = 'G2', cmd = 'J2', cd = 'K1', wp_n = None,
			dat0 = 'H2', dat1 = 'H3', dat2 = 'J1', dat3 = 'K2',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33'),
		),
		Resource(
			'floppy', 0,
			Subsignal('density', Pins('P6', dir = 'o')),
			Subsignal('motor', Pins('M5 H15', dir = 'o')),
			Subsignal('select', Pins('N5 G17', dir = 'o')),
			Subsignal('step_dir', Pins('P5', dir = 'o')),
			Subsignal('step', Pins('M3', dir = 'o')),
			Subsignal('wdata', Pins('N4', dir = 'o')),
			Subsignal('wgate', Pins('N3', dir = 'o')),
			Subsignal('side1', Pins('M1', dir = 'o')),
			Subsignal('index', Pins('M2', dir = 'i')),
			Subsignal('track0', Pins('N2', dir = 'i')),
			Subsignal('wp', Pins('P2', dir = 'i')),
			Subsignal('rdata', Pins('P1', dir = 'i')),
			Subsignal('diskchanged', Pins('R1', dir = 'i')),
			Attrs(IOSTANDARD = 'LVCMOS33'),
		),
	]

	connectors = [
		Connector('pmod', 0, 'G1 E1 C2 B1 - - F1 D1 B2 A1 - -'),
		Connector('pmod', 1, 'E2 D2 G4 J5 - - F3 E3 H4 H5 - -'),
		Connector('test', 0, 'T16 U16 W16 J19 K17 N19 N20 D20'),
	]


if __name__ == '__main__':
	from ...test.blinky import Blinky
	Mega65r3Platform(toolchain = 'yosys_nextpnr').build(
		Blinky(), do_build = True, do_program = False)



# SPDX-License-Identifier: BSD-2-Clause

from textwrap                           import dedent

from torii.build                        import Attrs, Clock, Connector, Pins, PinsN, Resource, Subsignal
from torii.build.run                    import BuildProducts
from torii.hdl.ir                       import Fragment
from torii.platform.resources.display   import Display7SegResource, VGAResource
from torii.platform.resources.interface import I2CResource, PS2Resource, UARTResource
from torii.platform.resources.memory    import SDRAMResource
from torii.platform.resources.user      import ButtonResources, LEDResources
from torii.platform.vendor.altera       import AlteraPlatform

__all__ = (
	'RZEasyFPGAA2_2Platform',
)

class RZEasyFPGAA2_2Platform(AlteraPlatform):
	device      = 'EP4CE6' # Cyclone IV 6K LEs
	package     = 'E22'    # EQFP 144 pins
	speed       = 'C8'
	default_clk = 'clk50'  # 50MHz builtin clock
	default_rst = 'rst'

	pretty_name = 'RZ-EasyFPGA A2.2'
	description = 'RZ-EasyFPGA Altera Cyclone IV Development Board'

	resources   = [
		# Clock
		Resource(
			'clk50', 0, Pins('23', dir = 'i'), Clock(50e6), Attrs(io_standard = '3.3-V LVTTL')
		),
		# Reset switch, located on the lower left of the board.
		Resource('rst', 0, PinsN('25', dir = 'i'), Attrs(io_standard = '3.3-V LVTTL')),
		# LEDs, located on the bottom of the board.
		*LEDResources(
			pins = '87 86 85 84', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		# Buttons, located on the bottom of the board, right of the LEDs.
		*ButtonResources(
			pins = '88 89 90 91', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		# Connections to the SKHynix RAM chip on board.
		SDRAMResource(
			0,
			clk = '43', cs_n = '72', we_n = '69', ras_n = '71', cas_n = '70',
			ba = '73 74', a = '76 77 80 83 68 67 66 65 64 60 75 59',
			dq = '28 30 31 32 33 34 38 39 54 53 52 51 50 49 46 44',
			dqm = '42 55', attrs = Attrs(io_standard = '3.3-V LVCMOS')
		),
		# VGA connector, located on the right of the board.
		VGAResource(
			0,
			r = '106', g = '105', b = '104',
			hs = '101', vs = '103'
		),
		# 4 digit 7 segment display, located on top of the board.
		Display7SegResource(
			0,
			a = '128', b = '121', c = '125', d = '129', e = '132', f = '126', g = '124', dp = '127',
			invert = True
		),
		Resource(
			'display_7seg_ctrl', 0,
			Subsignal('en', Pins('133 135 136 137', dir = 'o', invert = True)),
		),
		# PS2 port, located on upper right of the board.
		PS2Resource(0, clk = '119', dat = '120'),
		# LM75 temperature sensor
		I2CResource(0, scl = '112', sda = '113'),
		# AT24C08 EEPROM
		I2CResource(1, scl = '99', sda = '98'),
		# Buzzer
		Resource('buzzer', 0, PinsN('110', dir = 'o')),
		# Serial port, located above the VGA port.
		UARTResource(0, tx = '114', rx = '115'),
		# LCD connector, located above the 7 segment display.
		Resource(
			'lcd_hd44780', 0,
			Subsignal('rs', Pins('141', dir = 'o')),
			Subsignal('rw', Pins('138', dir = 'o')),
			Subsignal('e', Pins('143', dir = 'o')),
			Subsignal('d', Pins('142 1 144 3 2 10 7 11', dir = 'io')),
		),
		# IR receiver, located right of the buttons.
		Resource(
			'cir', 0,
			Subsignal('rx', Pins('100', dir = 'i'))
		),
	]

	connectors  = [
		# Located above the chip.
		Connector(
			'gpio', 0,
			'-   -   11  7   2   144 142 138 136 133 129 127 125 121 119 114 112 110 -  '
			'-   -   24  10  3   1   143 141 137 135 132 128 126 124 120 115 113 111 -  '
		),
		# Located right of the chip.
		Connector(
			'gpio', 1,
			'-   -   '
			'106 105 '
			'104 103 '
			'101 100 '
			'99  98  '
			'91  90  '
			'89  88  '
			'87  86  '
			'85  84  '
			'-   -   '
		),
		# Located below the chip.
		Connector(
			'gpio', 2,
			'30 32 34 39 43 46 50 52 54 58 60 65 67 71 73 75 77 83 -  -  - '
			'28 31 33 38 42 44 51 53 55 59 64 66 68 70 72 74 76 80 -  -  - '
		),
	]

	def toolchain_prepare(self, fragment: Fragment, name: str, **kwargs):
		overrides = {
			'add_settings': dedent('''
				set_global_assignment -name CYCLONEII_RESERVE_NCEO_AFTER_CONFIGURATION 'USE AS REGULAR IO'
			''')
		}
		return super().toolchain_prepare(fragment, name, **overrides, **kwargs)

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		quartus_pgm = environ.get('QUARTUS_PGM', 'quartus_pgm')
		with products.extract(f'{name}.sof') as bitstream_filename:
			check_call([
				quartus_pgm, '--haltcc', '--mode', 'JTAG',
				'--operation', 'P;' + bitstream_filename
			])


if __name__ == '__main__':
	from ...test.blinky import Blinky
	RZEasyFPGAA2_2Platform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-3-Clause

from os             import getenv
from pathlib        import Path
from shutil         import copy

import nox
from nox.sessions   import Session

ROOT_DIR  = Path(__file__).parent

BUILD_DIR = ROOT_DIR  / 'build'
CNTRB_DIR = ROOT_DIR  / 'contrib'
DOCS_DIR  = ROOT_DIR  / 'docs'
DIST_DIR  = BUILD_DIR / 'dist'

IN_CI           = getenv('GITHUB_WORKSPACE') is not None
ENABLE_COVERAGE = IN_CI or getenv('TORII_TEST_COVERAGE') is not None

# Default sessions to run
nox.options.sessions = (
	'test',
	'lint',
	'typecheck-mypy'
)

# Try to use `uv`, if not fallback to `virtualenv`
nox.options.default_venv_backend = 'uv|virtualenv'

@nox.session(reuse_venv = True)
def test(session: nox.Session) -> None:
	OUTPUT_DIR = BUILD_DIR / 'tests'
	OUTPUT_DIR.mkdir(parents = True, exist_ok = True)

	unittest_args = ('-m', 'unittest', 'discover', '-s', str(ROOT_DIR))

	session.install('-e', '.')

	# XXX(aki): Coverage is not quite possible yet
	if False and ENABLE_COVERAGE:
		session.log('Coverage support enabled')
		session.install('coverage')
		coverage_args = ('-m', 'coverage', 'run', '-p', f'--rcfile={ROOT_DIR / "pyproject.toml"}',)
		session.env['COVERAGE_CORE'] = 'sysmon'
	else:
		coverage_args = tuple[str]()

	with session.chdir(OUTPUT_DIR):
		session.log('Running test suite...')
		session.run('python', *coverage_args, *unittest_args, *session.posargs)

	# XXX(aki): Coverage is not quite possible yet
	if False and ENABLE_COVERAGE:
		session.log('Combining Coverage data..')
		session.run('python', '-m', 'coverage', 'combine')

		session.log('Generating XML Coverage report...')
		session.run('python', '-m', 'coverage', 'xml', f'--rcfile={ROOT_DIR / "pyproject.toml"}')

@nox.session(name = 'watch-docs', reuse_venv = True)
def watch_docs(session: Session) -> None:
	OUTPUT_DIR = BUILD_DIR / 'docs'

	session.install('-r', str(DOCS_DIR / 'requirements.txt'))
	session.install('sphinx-autobuild')
	session.install('-e', '.')

	session.run('sphinx-autobuild', str(DOCS_DIR), str(OUTPUT_DIR))

@nox.session(name = 'build-docs', reuse_venv = True)
def build_docs(session: Session) -> None:
	OUTPUT_DIR = BUILD_DIR / 'docs'

	session.install('-r', str(DOCS_DIR / 'requirements.txt'))
	session.install('-e', '.')

	session.run('sphinx-build', '-b', 'html', str(DOCS_DIR), str(OUTPUT_DIR))

@nox.session(name = 'build-docs-multiversion', reuse_venv = True)
def build_docs_multiversion(session: Session) -> None:
	OUTPUT_DIR = BUILD_DIR / 'mv-docs'

	redirect_index = (CNTRB_DIR / 'docs-redirect.html')

	session.install('-r', str(DOCS_DIR / 'requirements.txt'))
	session.install('-e', '.')

	# Workaround for sphinx-contrib/multiversion#58
	# Ask git for the list of tags matching `v*`, and sort them in reverse order by name
	git_tags: str = session.run(
		'git', 'tag', '-l', 'v*', '--sort=-v:refname',
		external = True, silent = True
	) # type: ignore
	# Split the tags and get the first, it *should* be the most recent
	latest = git_tags.splitlines()[0]

	# Build the multi-version docs
	session.run(
		'sphinx-multiversion', '-D', f'smv_latest_version={latest}', str(DOCS_DIR), str(OUTPUT_DIR)
	)

	session.log('Copying docs redirect...')
	# Copy the docs redirect index
	copy(redirect_index, OUTPUT_DIR / 'index.html')

	with session.chdir(OUTPUT_DIR):
		latest_link = Path('latest')
		docs_dev    = Path('main')
		docs_tag    = Path(latest)

		session.log('Copying needed GitHub pages files...')

		copy(docs_dev / 'CNAME', 'CNAME')
		copy(docs_dev / '.nojekyll', '.nojekyll')

		session.log('Creating symlink to latest docs...')
		# If the symlink exists, unlink it
		if latest_link.exists():
			latest_link.unlink()

		# Check to make sure the latest tag has some docs
		if docs_tag.exists():
			# Create a symlink from `/latest` to the latest tag
			latest_link.symlink_to(docs_tag)
		else:
			session.warn(f'Docs for {latest} did not seem to be built, using development docs instead')
			# Otherwise, link to `main`
			latest_link.symlink_to(docs_dev)

@nox.session(name = 'linkcheck-docs', reuse_venv = True)
def linkcheck_docs(session: Session) -> None:
	OUTPUT_DIR = BUILD_DIR / 'docs-linkcheck'

	session.install('-r', str(DOCS_DIR / 'requirements.txt'))
	session.install('-e', '.')

	session.run('sphinx-build', '-b', 'linkcheck', str(DOCS_DIR), str(OUTPUT_DIR))

@nox.session(name = 'typecheck-mypy', reuse_venv = True)
def typecheck_mypy(session: Session) -> None:
	OUTPUT_DIR = BUILD_DIR / 'typing' / 'mypy'
	OUTPUT_DIR.mkdir(parents = True, exist_ok = True)

	session.install('mypy')
	session.install('lxml')
	session.install('-e', '.')

	session.run(
		'mypy', '--non-interactive', '--install-types', '--pretty',
		'--disallow-any-generics',
		'--cache-dir', str((OUTPUT_DIR / '.mypy-cache').resolve()),
		'-p', 'torii_boards', '--html-report', str(OUTPUT_DIR.resolve())
	)

@nox.session(name = 'typecheck-pyright', reuse_venv = True)
def typecheck_pyright(session: Session) -> None:
	OUTPUT_DIR = BUILD_DIR / 'typing' / 'pyright'
	OUTPUT_DIR.mkdir(parents = True, exist_ok = True)

	session.install('pyright')
	session.install('-e', '.')

	with (OUTPUT_DIR / 'pyright.log').open('w') as f:
		session.run('pyright', *session.posargs, stdout = f)

@nox.session(reuse_venv = True)
def lint(session: nox.Session) -> None:
	session.install('flake8')
	session.install('ruff')

	session.run(
		'flake8', '--config', str((CNTRB_DIR / '.flake8').resolve()),
		'./torii_boards', './tests', './docs'
	)
	session.run('ruff', 'check', './torii_boards', './tests', './docs')


@nox.session(reuse_venv = True)
def dist(session: Session) -> None:
	session.install('build')

	session.run('python', '-m', 'build', '-o', str(DIST_DIR))



# SPDX-License-Identifier: BSD-2-Clause

from .atlys        import AtlysPlatform
from .numato_mimas import NumatoMimasPlatform
from .sk_xc6slx9   import SK_XC6SLX9Platform

__all__ = (
	'AtlysPlatform',
	'NumatoMimasPlatform',
	'SK_XC6SLX9Platform',
)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                        import Attrs, Clock, Connector, Pins, PinsN, Resource, Subsignal
from torii.build.run                    import BuildProducts
from torii.platform.resources.display   import VGAResource
from torii.platform.resources.interface import SPIResource, UARTResource
from torii.platform.resources.memory    import SDCardResources, SDRAMResource
from torii.platform.resources.user      import ButtonResources, LEDResources, SwitchResources
from torii.platform.vendor.altera       import AlteraPlatform

__all__ = (
	'MisterPlatform',
)

# The MiSTer platform is built around the DE10-Nano; if you update one you should update the other.
class MisterPlatform(AlteraPlatform):
	device      = '5CSEBA6' # Cyclone V 110K LEs
	package     = 'U23'     # UBGA-484
	speed       = 'I7'
	default_clk = 'clk50'

	pretty_name = 'MiSTer FPGA'
	description = 'terasIC DE10-Nano based MiSTer FPGA Platform'

	resources   = [
		Resource(
			'clk50', 0, Pins('V11', dir = 'i'), Clock(50e6), Attrs(io_standard = '3.3-V LVTTL')
		),
		Resource(
			'clk50', 1, Pins('Y13', dir = 'i'), Clock(50e6), Attrs(io_standard = '3.3-V LVTTL')
		),
		Resource(
			'clk50', 2, Pins('E11', dir = 'i'), Clock(50e6), Attrs(io_standard = '3.3-V LVTTL')
		),
		*LEDResources(
			pins = 'W15 AA24 V16 V15 AF26 AE26 Y16 AA23',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		*ButtonResources(
			pins = 'AH17 AH16', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		*SwitchResources(
			pins = 'Y24 W24 W21 W20',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		# Arduino header
		UARTResource(
			0,
			rx = 'AG13', tx = 'AF13',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		# LTC2308 analogue-to-digital converter
		SPIResource(
			0,
			cs_n = 'U9', clk = 'V10', copi = 'AC4', cipo = 'AD4',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		# ADV7513 HDMI transmitter
		# Note this has a lot of input formats for tx_d, but this defaults to RGB24
		Resource(
			'adv7513', 0,
			Subsignal('tx_d_r', Pins('AD12 AE12 W8 Y8 AD11 AD10 AE11 Y5',  dir = 'o')),
			Subsignal('tx_d_g', Pins('AF10 Y4 AE9 AB4 AE7 AF6 AF8 AF5', dir = 'o')),
			Subsignal('tx_d_b', Pins('AE4 AH2 AH4 AH5 AH6 AG6 AF9 AE8', dir = 'o')),
			Subsignal('tx_clk', Pins('AG5', dir = 'o')),
			Subsignal('tx_de', Pins('AD19', dir = 'o')),
			Subsignal('tx_hs', Pins('T8', dir = 'o')),
			Subsignal('tx_vs', Pins('V13', dir = 'o')),
			Subsignal('tx_int', Pins('AF11', dir = 'i')),
			Subsignal('i2s0', Pins('T13', dir = 'o')),
			Subsignal('mclk', Pins('U11', dir = 'o')),
			Subsignal('lrclk', Pins('T11', dir = 'o')),
			Subsignal('sclk', Pins('T12', dir = 'o')),
			Subsignal('scl', Pins('U10', dir = 'io')),
			Subsignal('sda', Pins('AA4', dir = 'io')),
			Attrs(io_standard = '3.3-V LVTTL')
		),
		# MiSTer SDRAM Board (required)
		# https://github.com/MiSTer-devel/Hardware_MiSTer/blob/master/releases/sdram_xs_2.2.pdf
		SDRAMResource(
			0,
			clk = '20', cs_n = '33', we_n = '27', ras_n = '32', cas_n = '31',
			ba = '34 35', a = '37 38 39 40 28 25 26 23 24 21 36 22 19',
			dq = '1 2 3 4 5 6 7 8 18 17 16 15 14 13 9 10',
			dqm = '', conn = ('gpio', 0), attrs = Attrs(io_standard = '3.3-V LVCMOS')
		),
		# MiSTer I/O Board (optional, but highly recommended)
		# https://github.com/MiSTer-devel/Hardware_MiSTer/blob/master/releases/iobrd_6.0.pdf
		Resource(
			'power_led', 0, PinsN('1', dir = 'o', conn = ('gpio', 1)), Attrs(io_standard = '3.3-V LVTTL')
		),
		Resource(
			'disk_led', 0, PinsN('3', dir = 'o', conn = ('gpio', 1)), Attrs(io_standard = '3.3-V LVTTL')
		),
		Resource(
			'user_led', 0, PinsN('5', dir = 'o', conn = ('gpio', 1)), Attrs(io_standard = '3.3-V LVTTL')
		),
		Resource(
			'reset_switch', 0, PinsN('17', dir = 'i', conn = ('gpio', 1)), Attrs(io_standard = '3.3-V LVTTL')
		),
		Resource(
			'osd_switch', 0, PinsN('13', dir = 'i', conn = ('gpio', 1)), Attrs(io_standard = '3.3-V LVTTL')
		),
		Resource(
			'user_switch', 0, PinsN('15', dir = 'i', conn = ('gpio', 1)), Attrs(io_standard = '3.3-V LVTTL')
		),
		Resource(
			'audio', 0,
			Subsignal('l', Pins('2', dir = 'o', conn = ('gpio', 1))),
			Subsignal('r', Pins('7', dir = 'o', conn = ('gpio', 1))),
			Attrs(io_standard = '3.3-V LVTTL')
		),
		Resource('toslink', 0, Pins('9', dir = 'o', conn = ('gpio', 1))),
		*SDCardResources(
			0,
			clk = '13', cmd = '8',
			dat0 = '16', dat1 = '18', dat2 = '4', dat3 = '6',
			conn = ('gpio', 1), attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		# The schematic is difficult to understand here...
		VGAResource(
			0,
			r = '28 32 34 36 38 40',
			g = '27 31 33 35 37 39',
			b = '21 23 25 26 24 24',
			hs = '20', vs = '19',
			conn = ('gpio', 1),
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		)
	]

	connectors  = [
		# Located on the top of the board, above the chip.
		Connector(
			'gpio', 0,
			'V12  E8   W12  D11  D8   AH13 AF7  AH14 AF4  AH3  '
			'-    -    AD5  AG14 AE23 AE6  AD23 AE24 D12  AD20 '
			'C12  AD17 AC23 AC22 Y19  AB23 AA19 W11  -    -    '
			'AA18 W14  Y18  Y17  AB25 AB26 Y11  AA26 AA13 AA11 '
		),
		# Located on the bottom of the board.
		Connector(
			'gpio', 1,
			'Y15  AC24 AA15 AD26 AG28 AF28 AE25 AF27 AG26 AH27 '
			'-    -    AG25 AH26 AH24 AF25 AG23 AF23 AG24 AH22 '
			'AH21 AG21 AH23 AA20 AF22 AE22 AG20 AF21 -    -    '
			'AG19 AH19 AG18 AH18 AF18 AF20 AG15 AE20 AE19 AE17 '
		),
		Connector(
			'arduino', 0,
			'AG13 AF13 AG10 AG9  U14  U13  AG8  AH8  '
			'AF17 AE15 AF15 AG16 AH11 AH12 AH9  AG11 '
			'AH7  -    -    -    -    -    -    -    '
		),
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		quartus_pgm = environ.get('QUARTUS_PGM', 'quartus_pgm')
		with products.extract(f'{name}.sof') as bitstream_filename:
			# The @2 selects the second device in the JTAG chain, because this chip
			# puts the ARM cores first.
			check_call([
				quartus_pgm, '--haltcc', '--mode', 'JTAG',
				'--operation', 'P;' + bitstream_filename + '@2'
			])


if __name__ == '__main__':
	from ...test.blinky import Blinky
	MisterPlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                        import Attrs, Clock, Connector, Pins, Resource
from torii.build.run                    import BuildProducts
from torii.platform.resources.display   import Display7SegResource, VGAResource
from torii.platform.resources.interface import PS2Resource
from torii.platform.resources.memory    import SDCardResources, SDRAMResource
from torii.platform.resources.user      import ButtonResources, LEDResources, SwitchResources
from torii.platform.vendor.altera       import AlteraPlatform

__all__ = (
	'DE0CVPlatform',
)

class DE0CVPlatform(AlteraPlatform):
	device      = '5CEBA4' # Cyclone V 49K LEs
	package     = 'F23'    # FBGA-484
	speed       = 'C7'
	default_clk = 'clk50'

	pretty_name = 'DE0-CV'
	description = 'terasIC DE0-CV Altera Cyclone V Development Board'

	resources   = [
		Resource(
			'clk50', 0, Pins('M9', dir = 'i'),  Clock(50e6), Attrs(io_standard = '3.3-V LVTTL')
		),
		Resource(
			'clk50', 1, Pins('H13', dir = 'i'), Clock(50e6), Attrs(io_standard = '3.3-V LVTTL')
		),
		Resource(
			'clk50', 2, Pins('E10', dir = 'i'), Clock(50e6), Attrs(io_standard = '3.3-V LVTTL')
		),
		Resource(
			'clk50', 3, Pins('V15', dir = 'i'), Clock(50e6), Attrs(io_standard = '3.3-V LVTTL')
		),

		*LEDResources(
			pins = 'AA2 AA1 W2 Y3 N2 N1 U2 U1 L2 L1', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		*ButtonResources(
			pins = 'U7 W9 M7 M6', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		*SwitchResources(
			pins = 'U13 V13 T13 T12 AA15 AB15 AA14 AA13 AB13 AB12',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		Display7SegResource(
			0,
			a = 'U21', b = 'V21', c = 'W22', d = 'W21', e = 'Y22', f = 'Y21', g = 'AA22', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		Display7SegResource(
			1,
			a = 'AA20', b = 'AB20', c = 'AA19', d = 'AA18', e = 'AB18', f = 'AA17', g = 'U22', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		Display7SegResource(
			2,
			a = 'Y19', b = 'AB17', c = 'AA10', d = 'Y14', e = 'V14', f = 'AB22', g = 'AB21', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		Display7SegResource(
			3,
			a = 'Y16', b = 'W16', c = 'Y17', d = 'V16', e = 'U17', f = 'V18', g = 'V19', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		Display7SegResource(
			4,
			a = 'U20', b = 'Y20', c = 'V20', d = 'U16', e = 'U15', f = 'Y15', g = 'P9', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		Display7SegResource(
			5,
			a = 'N9', b = 'M8', c = 'T14', d = 'P14', e = 'C1', f = 'C2', g = 'W19', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		VGAResource(
			0,
			r = 'A9 B10 C9 A5',
			g = 'L7 K7 J7 J8',
			b = 'B6 B7 A8 A7',
			hs = 'H8', vs = 'G8',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		PS2Resource(
			0, # Keyboard
			clk = 'D3', dat = 'G2', attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		PS2Resource(
			1, # Mouse
			clk = 'E2', dat = 'G1', attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		*SDCardResources(
			0,
			clk = 'H11', cmd = 'B11', dat0 = 'K9', dat1 = 'D12', dat2 = 'E12', dat3 = 'C11',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		SDRAMResource(
			0,
			clk = 'AB11', cke = 'R6', cs_n = 'U6', we_n = 'AB5', ras_n = 'AB6', cas_n = 'V6',
			ba = 'T7 AB7', a = 'W8 T8 U11 Y10 N6 AB10 P12 P7 P8 R5 U8 P6 R7',
			dq = 'Y9 T10 R9 Y11 R10 R11 R12 AA12 AA9 AB8 AA8 AA7 V10 V9 U10 T9', dqm = 'U12 N8',
			attrs = Attrs(io_standard = '3.3-V LVCMOS')
		),
	]
	connectors  = [
		Connector(
			'j', 1,
			'N16 B16 M16 C16 D17 K20 K21 K22 M20 M21 '
			'-   -   N21 R22 R21 T22 N20 N19 M22 P19 '
			'L22 P17 P16 M18 L18 L17 L19 K17 -   -   '
			'K19 P18 R15 R17 R16 T20 T19 T18 T17 T15 '
		),
		Connector(
			'j', 2,
			'H16 A12 H15 B12 A13 B13 C13 D13 G18 G17 '
			'-   -   H18 J18 J19 G11 H10 J11 H14 A15 '
			'J13 L8  A14 B15 C15 E14 E15 E16 -   -   '
			'F14 F15 F13 F12 G16 G15 G13 G12 J17 K16 '
		),
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		quartus_pgm = environ.get('QUARTUS_PGM', 'quartus_pgm')
		with products.extract(f'{name}.sof') as bitstream_filename:
			check_call([
				quartus_pgm, '--haltcc', '--mode', 'JTAG',
				'--operation', 'P;' + bitstream_filename
			])


if __name__ == '__main__':
	from ...test.blinky import Blinky
	DE0CVPlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                        import (
	Attrs, Clock, Connector, DiffPairs, DiffPairsN, Pins, PinsN, Resource, Subsignal
)
from torii.build.run                    import BuildPlan, BuildProducts
from torii.hdl.ir                       import Fragment
from torii.platform.resources.interface import DirectUSBResource, UARTResource
from torii.platform.resources.memory    import SDRAMResource
from torii.platform.resources.user      import ButtonResources, LEDResources
from torii.platform.vendor.lattice.ecp5 import ECP5Platform

__all__ = (
	'Supercon19BadgePlatform',
)

class Supercon19BadgePlatform(ECP5Platform):
	device      = 'LFE5U-45F'
	package     = 'BG381'
	speed       = '8'
	default_clk = 'clk8'

	pretty_name = 'Supercon 19 Badge'
	description = 'Lattice ECP5-45F Based Hackaday Supercon 2019 Badge'

	# The badge's LEDs are wired in a non-straightforward way. Here, the
	# LEDResources represent each of the common anodes of a collection of RGB LEDs.
	# A single one of their cathodes defaults to connected via a FET; the other
	# cathodes are normally-off. Accordingly, they act as normal single-color LEDs
	# unless the cathode signals are explicitly driven.
	#
	# The LEDs on the badge were meant to be a puzzle; so each cathode signal
	# corresponds to a different color on each LED. This means there's no
	# straightforward way of creating pin definitions; you'll need to use the
	# following mapping to find the cathode pin number below.
	led_cathode_mappings = [
		{'r': 0, 'g': 1, 'b': 2}, # LED1: by default, red
		{'r': 2, 'g': 1, 'b': 0}, # LED2: by default, blue
		{'r': 0, 'g': 1, 'b': 2}, # LED3: by default, red
		{'r': 2, 'g': 1, 'b': 0}, # LED4: by default, blue
		{'r': 0, 'g': 1, 'b': 2}, # LED5: by default, red
		{'r': 2, 'g': 1, 'b': 0}, # LED6: by default, blue
		{'r': 2, 'g': 0, 'b': 1}, # LED7: by default, green
		{'r': 0, 'g': 1, 'b': 2}, # LED8: by default, red
		{'r': 0, 'g': 1, 'b': 2}, # LED9: by default, red
		# Note: [LED10 is disconnected by default; bridge R60 to make things work]
		{'r': 2, 'g': 1, 'b': 0}, # LED10: by default, blue
		{'r': 1, 'g': 0, 'b': 2}, # LED11: by default, green
	]

	resources   = [
		Resource('clk8', 0, Pins('U18'), Clock(8e6), Attrs(IO_TYPE = 'LVCMOS33')),
		# Used to trigger FPGA reconfiguration.
		Resource('program', 0, PinsN('R1'), Attrs(IO_TYPE = 'LVCMOS33')),
		# See note above for LED anode/cathode information.
		# Short version is: these work as normal LEDs until you touch their cathodes.
		*LEDResources(
			pins = 'E3 D3 C3 C4 C2 B1 B20 B19 A18 K20 K19',
			attrs = Attrs(IO_TYPE = 'LVCMOS33')
		),
		Resource('led_cathodes', 0, Pins('P19 L18 K18'), Attrs(IO_TYPE = 'LVCMOS33')),
		UARTResource(0, rx = 'U2', tx = 'U1', attrs = Attrs(IO_TYPE = 'LVCMOS33')),
		DirectUSBResource(
			0,
			d_p = 'F3', d_n = 'G3', pullup = 'E4', vbus_valid = 'F4',
			attrs = Attrs(IO_TYPE = 'LVCMOS33')
		),
		# Buttons, with semantic aliases.
		*ButtonResources(
			pins = 'G2 F2 F1 C1 E1 D2 D1 E2',
			attrs = Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'UP')
		),
		Resource(
			'keypad', 0,
			Subsignal('left', Pins('G2', dir = 'i')),
			Subsignal('right', Pins('F2', dir = 'i')),
			Subsignal('up', Pins('F1', dir = 'i')),
			Subsignal('down', Pins('C1', dir = 'i')),
			Subsignal('start', Pins('E1', dir = 'i')),
			Subsignal('select', Pins('D2', dir = 'i')),
			Subsignal('a', Pins('D1', dir = 'i')),
			Subsignal('b', Pins('E2', dir = 'i')),
			Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'UP')
		),
		# TODO(aki): Replace with `HDMIResource` when merged + released in Torii
		Resource(
			'hdmi', 0,
			Subsignal('clk', DiffPairsN('P20', 'R20'), Attrs(IO_TYPE = 'TMDS_33')),
			Subsignal('d', DiffPairs('N19 L20 L16', 'N20 M20 L17'), Attrs(IO_TYPE = 'TMDS_33')),
			Subsignal('hpd', PinsN('R18'), Attrs(IO_TYPE = 'LVCMOS33')), # Also called HDMI_HEAC_n
			Subsignal('hdmi_heac_p', PinsN('T19'), Attrs(IO_TYPE = 'LVCMOS33')),
			Attrs(DRIVE = '4'),
		),
		# TODO(aki): Replace with LCDResource
		Resource(
			'lcd', 0,
			Subsignal('db', Pins('J3 H1 K4 J1 K3 K2 L4 K1 L3 L2 M4 L1 M3 M1 N4 N2 N3 N1')),
			Subsignal('rd', Pins('P2')),
			Subsignal('wr', Pins('P4')),
			Subsignal('rs', Pins('P1')),
			Subsignal('cs', Pins('P3')),
			Subsignal('id', Pins('J4')),
			Subsignal('rst', Pins('H2')),
			Subsignal('fmark', Pins('G1')),
			Subsignal('blen', Pins('P5')),
			Attrs(IO_TYPE = 'LVCMOS33')
		),
		# XXX(aki):
		# We can't replace this with `SPIFlashResource` because it needs a `clk` signal
		# And the SPI Clock needs to be done via the `USRMCLK` block in the ECP5.
		Resource(
			'spi_flash', 0,
			Subsignal('cs', PinsN('R2')),
			Subsignal('copi', Pins('W2')),
			Subsignal('cipo', Pins('V2')),
			Subsignal('wp', Pins('Y2')),
			Subsignal('hold', Pins('W1')),
			Attrs(IO_TYPE = 'LVCMOS33')
		),
		# XXX(aki):
		# We can't replace this with `SPIFlashResource` because it needs a `clk` signal
		# And the SPI Clock needs to be done via the `USRMCLK` block in the ECP5.
		Resource(
			'spi_flash_4x', 0,
			Subsignal('cs', PinsN('R2')),
			Subsignal('dq', Pins('W2 V2 Y2 W1')),
			Attrs(IO_TYPE = 'LVCMOS33')
		),
		# TODO(aki): Replace with `SPIResource` when it's QSPI compatible
		# SPI-connected PSRAM.
		Resource(
			'spi_psram_4x', 0,
			Subsignal('cs', PinsN('D20')),
			Subsignal('clk', Pins('E20')),
			Subsignal('dq', Pins('E19 D19 C20 F19'), Attrs(PULLMODE = 'UP')),
			Attrs(IO_TYPE = 'LVCMOS33', SLEWRATE = 'SLOW')
		),
		# TODO(aki): Replace with `SPIResource` when it's QSPI compatible
		Resource(
			'spi_psram_4x', 1,
			Subsignal('cs', PinsN('F20')),
			Subsignal('clk', Pins('J19')),
			Subsignal('dq', Pins('J20 G19 G20 H20'), Attrs(PULLMODE = 'UP')),
			Attrs(IO_TYPE = 'LVCMOS33', SLEWRATE = 'SLOW')
		),
		SDRAMResource(
			0,
			clk = 'D11', cke = 'C11', cs_n = 'C7', we_n = 'B6', ras_n = 'D6', cas_n = 'A6',
			ba = 'A7 C8', a = 'A8 D9 C9 B9 C14 E17 A12 B12 H17 G18 B8 A11 B11',
			dq = 'C5 B5 A5 C6 B10 C10 D10 A9', dqm = 'A10',
			attrs = Attrs(IO_TYPE = 'LVCMOS33', SLEWRATE = 'FAST')
		)
	]

	connectors = [
		Connector('pmod', 0, 'A15 C16 A14 D16 B15 C15 A13 B13'),
		Connector(
			'cartridge', 0,
			'-   -   -   -   -   -   -   -   C5  B5  A5  C6  B6  A6  D6  C7  A7  C8  B8 '
			'A8  D9  C9  B9  A9  D10 C10 B10 A10 D11 C11 B11 A11 G18 H17 B12 A12 E17 C14'
		),
		# SAO connectors names are compliant with the, erm, SAO 1.69 X-TREME standard.
		# See: https://hackaday.com/2019/03/20/introducing-the-shitty-add-on-v1-69bis-standard/
		Connector('sao', 0, {
			'sda': 'B3', 'scl': 'B2', 'gpio0': 'A2',
			'gpio1': 'A3', 'gpio2': 'B4', 'gpio3': 'A4'
		}),
		Connector('sao', 1, {
			'sda': 'A16', 'scl': 'B17', 'gpio0': 'B18',
			'gpio1': 'A17', 'gpio2': 'B16', 'gpio3': 'C17'
		})
	]

	def toolchain_prepare(self, fragment: Fragment, name: str, **kwargs) -> BuildPlan:
		overrides = dict(ecppack_opts = '--compress --freq 38.8')
		overrides.update(kwargs)
		return super().toolchain_prepare(fragment, name, **overrides, **kwargs)

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		dfu_util = environ.get('DFU_UTIL', 'dfu-util')
		with products.extract(f'{name}.bit') as bitstream_filename:
			check_call([dfu_util, '-d', '1d50:614b', '-a', '0', '-D', bitstream_filename])


if __name__ == '__main__':
	from ...test.blinky import Blinky
	Supercon19BadgePlatform().build(Blinky(), do_program = True)
